"""
This backend is based on the 2D graphics package Grace (available at
http://plasma-gate.weizmann.ac.il/Grace). The connection with Python is
handled by the grace_np.py module written by Michael Haggerty. This module
is available in the pygrace module by Mike McKerns (see
http://www.its.caltech.edu/~mmckerns/software.html). Only curve plotting is
currently available. However, histograms and pie charts might be added in
the future. The Grace backend can be used by

  python somefile.py --SCITOOLS_easyviz_backend grace

or one can specify the backend in the SciTools configuration file
scitools.cfg under the [easyviz] section

  [easyviz]
  backend = grace

and then

  from scitools.std import *

or if just easyviz is needed

  from scitools.easyviz import *

REQUIREMENTS:

Grace
pygrace

Known issues:

- To be able to store a copy of a plot with the hardcopy command, one must
  disable safe mode with the -nosafe option when running Grace. One can do
  this by adding the following line to GraceProcess.__init__ in grace_np.py
  (e.g. at line 130):

    cmd = cmd + ('-nosafe',)

- PDF output does not seem to work (at least on my configuration).

Tip:

- One way to create axes at arbitrary positions is to use the axes command
  together with the viewport optional argument. In the Grace backend this
  argument should be given as the list [xmin,ymin,xmax,ymax] where (xmin,ymin)
  is the lower-left corner and (xmax,ymax) is the upper-right corner. An 
  example is provided next.

    >>> x = seq(-3,3,0.1)
    >>> subplot(2,2,3)
    >>> plot(x,x**2,'ro:2',x,8*sin(2*pi*x),'g',x,x**3,'mx')
    [<scitools.easyviz.common.Line object at 0xb15645ac>]
    >>> subplot(2,2,4)
    >>> plot(x,x**3,'r+',title='subplot(2,2,4)')
    [<scitools.easyviz.common.Line object at 0xb31596ac>]
    >>> ax = axes(viewport=[0.1, 0.55, 0.9, 0.9])
    >>> plot(ax,x,cos(x),'bd--',axis=[-2,4,-1,2]')

"""

from __future__ import division

from common import *
from scitools.globaldata import DEBUG, VERBOSE
from scitools.misc import test_if_module_exists 

if test_if_module_exists('pygrace', msg='You need to install the pygrace package from http://www.cacr.caltech.edu/~mmckerns/pygrace.html (Not PyGrace from sourceforge!) and the grace program.', abort=False):
    from pygrace import grace_np
else:
    raise ImportError('Cannot import grace_np from pygrace')


class GraceBackend(BaseClass):
    def __init__(self):
        BaseClass.__init__(self)
        self._init()
        
    def _init(self, *args, **kwargs):
        """Perform initialization that is special for this backend."""
        
        # Set docstrings of all functions to the docstrings of BaseClass
        # The exception is if something is very different
        
        
        self.figure(self.getp('curfig'))

        # conversion tables for format strings:
        self._markers = {
            '': 0,   # no marker
            '.': 0,  # dot --> no marker
            'o': 1,  # circle
            'x': 9,  # cross
            '+': 8,  # plus sign
            '*': 10, # asterisk
            's': 2,  # square
            'd': 3,  # diamond
            '^': 4,  # triangle (up)
            'v': 6,  # triangle (down)
            '<': 5,  # triangle (left)
            '>': 7,  # triangle (right)
            'p': 2,  # pentagram --> square
            'h': 3,  # hexagram --> diamond
            }
                        
        self._colors = {
            '': 4,   # no color --> blue
            'r': 2,  # red
            'g': 3,  # green
            'b': 4,  # blue
            'c': 9,  # cyan
            'm': 10, # magenta
            'y': 5,  # yellow
            'k': 1,  # black
            'w': 0,  # white
            }
        
        self._line_styles = {
            '': None, # no line 
            '-': 1,   # solid line
            ':': 2,   # dotted line
            '-.': 6,  # dash-dot line
            '--': 4,  # dashed line
            }

        # convert table for colorbar location:
        self._colorbar_locations = {
            'North': None,
            'South': None,
            'East': None,
            'West': None,
            'NorthOutside': None,
            'SouthOutside': None,
            'EastOutside': None,
            'WestOutside': None,
            }

        if DEBUG:
            print "Setting backend standard variables"
            for disp in 'self._markers self._colors self._line_styles'.split():
                print disp, eval(disp)

    def _set_scale(self, ax):
        """Set linear or logarithmic (base 10) axis scale."""
        if DEBUG:
            print "Setting scales"
        scale = ax.getp('scale')
        if scale == 'loglog':
            # use logarithmic scale on both x- and y-axis
            self._g('xaxes scale logarithmic')
            self._g('yaxes scale logarithmic')
        elif scale == 'logx':
            # use logarithmic scale on x-axis and linear scale on y-axis
            self._g('xaxes scale logarithmic')
            self._g('yaxes scale normal')
            pass
        elif scale == 'logy':
            # use linear scale on x-axis and logarithmic scale on y-axis
            self._g('xaxes scale normal')
            self._g('yaxes scale logarithmic')
        elif scale == 'linear':
            # use linear scale on both x- and y-axis
            self._g('xaxes scale normal')
            self._g('yaxes scale normal')

    def _set_labels(self, ax):
        """Add text labels for x-, y-, and z-axis."""
        if DEBUG:
            print "Setting labels"
        xlabel = ax.getp('xlabel')
        ylabel = ax.getp('ylabel')
        zlabel = ax.getp('zlabel')
        self._g('xaxis label "%s"' % xlabel)
        self._g('yaxis label "%s"' % ylabel)
        if zlabel:
            # add a text label on z-axis
            pass
        
    def _set_title(self, ax):
        """Add a title at the top of the axis."""
        if DEBUG:
            print "Setting title"
        title = self._fix_latex(ax.getp('title'))
        self._g('subtitle "%s"' % title)  # set title
    
    def _set_limits(self, ax):
        """Set axis limits in x, y, and z direction."""
        if DEBUG:
            print "Setting axis limits"
        mode = ax.getp('mode')
        if mode == 'auto':
            # let plotting package set 'nice' axis limits in the x, y,
            # and z direction. If this is not automated in the plotting
            # package, one can use the following limits:
            #xmin, xmax, ymin, ymax, zmin, zmax = ax.get_limits()
            self._g('autoscale xaxes')
            self._g('autoscale yaxes')
        elif mode == 'manual':
            # (some) axis limits are frozen
            xmin = ax.getp('xmin')
            xmax = ax.getp('xmax')
            if xmin is not None and xmax is not None:
                # set x-axis limits
                self._g('world xmin %s' % xmin)
                self._g('world xmax %s' % xmax)
            else:
                # let plotting package set x-axis limits or use
                #xmin, xmax = ax.getp('xlim')
                self._g('autoscale xaxes')

            ymin = ax.getp('ymin')
            ymax = ax.getp('ymax')
            if ymin is not None and ymax is not None:
                # set y-axis limits
                self._g('world ymin %s' % ymin)
                self._g('world ymax %s' % ymax)
            else:
                # let plotting package set y-axis limits or use
                #ymin, ymax = ax.getp('ylim')
                self._g('autoscale yaxes')

            zmin = ax.getp('zmin')
            zmax = ax.getp('zmax')
            if zmin is not None and zmax is not None:
                # set z-axis limits
                pass
            else:
                # let plotting package set z-axis limits or use
                #zmin, zmax = ax.getp('zlim')
                pass
        elif mode == 'tight':
            # set the limits on the axis to the range of the data. If
            # this is not automated in the plotting package, one can
            # use the following limits:
            xmin, xmax, ymin, ymax, zmin, zmax = ax.get_limits()
            self._g('world xmin %s' % xmin)
            self._g('world xmax %s' % xmax)
            self._g('world ymin %s' % ymin)
            self._g('world ymax %s' % ymax)
        elif mode == 'fill':
            # not sure about this
            pass

    def _set_position(self, ax):
        """Set axes position."""
        rect = ax.getp('viewport')
        if rect:
            # axes position is defined. In Matlab rect is defined as
            # [left,bottom,width,height], where the four parameters are
            # location values between 0 and 1 ((0,0) is the lower-left
            # corner and (1,1) is the upper-right corner).
            # NOTE: This can be different in the plotting package.
            # In Grace the position is specified as [xmin,ymin,xmax,ymax],
            # where (xmin,ymin) is the lower-left corner and (xmax,ymin)
            # is the top-right corner.
            xmin, ymin, xmax, ymax = rect
            if not ax.getp('pth'):
                self._g('view %s, %s, %s, %s' % (xmin, ymin, xmax, ymax))

    def _set_daspect(self, ax):
        """Set data aspect ratio."""
        if ax.getp('daspectmode') == 'manual':
            dar = ax.getp('daspect')  # dar is a list (len(dar) is 3).
            pass
        else:
            # daspectmode is 'auto'. Plotting package handles data
            # aspect ratio automatically.
            pass
        
    def _set_axis_method(self, ax):
        method = ax.getp('method')
        if method == 'equal':
            # tick mark increments on the x-, y-, and z-axis should
            # be equal in size.
            pass
        elif method == 'image':
            # same effect as axis('equal') and axis('tight')
            pass
        elif method == 'square':
            # make the axis box square in size
            pass
        elif method == 'normal':
            # full size axis box
            pass
        elif method == 'vis3d':
            # freeze data aspect ratio when rotating 3D objects
            pass

    def _set_coordinate_system(self, ax):
        """
        Use either the default Cartesian coordinate system or a
        matrix coordinate system.
        """
        
        direction = ax.getp('direction')
        if direction == 'ij':
            # Use matrix coordinates. The origin of the coordinate
            # system is the upper-left corner. The i-axis should be
            # vertical and numbered from top to bottom, while the j-axis
            # should be horizontal and numbered from left to right.
            self._g('yaxes invert on')
        elif direction == 'xy':
            # use the default Cartesian axes form. The origin is at the
            # lower-left corner. The x-axis is vertical and numbered
            # from left to right, while the y-axis is vertical and
            # numbered from bottom to top.
            self._g('yaxes invert off')

    def _set_box(self, ax):
        """Turn box around axes boundary on or off."""
        if DEBUG:
            print "Setting box"
        if ax.getp('box'):
            # display box 
            pass
        else:
            # do not display box
            pass
        
    def _set_grid(self, ax):
        """Turn grid lines on or off."""
        if DEBUG:
            print "Setting grid"
        if ax.getp('grid'):
            # turn grid lines on
            self._g('xaxis tick major linestyle 2')
            self._g('xaxis tick major grid on')
            self._g('yaxis tick major linestyle 2')
            self._g('yaxis tick major grid on')
        else:
            # turn grid lines off
            self._g('xaxis tick major linestyle 1')
            self._g('xaxis tick major grid off')
            self._g('yaxis tick major linestyle 1')
            self._g('yaxis tick major grid off')

    def _set_hidden_line_removal(self, ax):
        """Turn on/off hidden line removal for meshes."""
        if DEBUG:
            print "Setting hidden line removal"
        if ax.getp('hidden'):
            # turn hidden line removal on
            pass
        else:
            # turn hidden line removal off
            pass

    def _set_colorbar(self, ax):
        """Add a colorbar to the axis."""
        if DEBUG:
            print "Setting colorbar"
        cbar = ax.getp('colorbar')
        if cbar.getp('visible'):
            # turn on colorbar
            cbar_title = cbar.getp('cbtitle')
            cbar_location = self._colorbar_locations[cbar.getp('cblocation')]
            # ...
        else:
            # turn off colorbar
            pass

    def _set_caxis(self, ax):
        """Set the color axis scale."""
        if DEBUG:
            print "Setting caxis"
        if ax.getp('caxismode') == 'manual':
            cmin, cmax = ax.getp('caxis')
            # NOTE: cmin and cmax might be None:
            if cmin is None or cmax is None:
                cmin, cmax = [0,1]
            # set color axis scaling according to cmin and cmax
            pass
        else:
            # use autoranging for color axis scale
            pass

    def _set_colormap(self, ax):
        """Set the colormap."""
        if DEBUG:
            print "Setting colormap"
        cmap = ax.getp('colormap')
        # cmap is plotting package dependent

    def _set_view(self, ax):
        """Set viewpoint specification."""
        if DEBUG:
            print "Setting view"
        cam = ax.getp('camera')
        view = cam.getp('view')
        if view == 2:
            # setup a default 2D view
            pass
        elif view == 3:
            az = cam.getp('azimuth')
            el = cam.getp('elevation')
            if az is None or el is None:
                # azimuth or elevation is not given. Set up a default
                # 3D view (az=-37.5 and el=30 is the default 3D view in
                # Matlab).
                pass
            else:
                # set a 3D view according to az and el
                pass
            
            if cam.getp('cammode') == 'manual':
                # for advanced camera handling:
                roll = cam.getp('camroll')
                zoom = cam.getp('camzoom')
                dolly = cam.getp('camdolly')
                target = cam.getp('camtarget')
                position = cam.getp('campos')
                up_vector = cam.getp('camup')
                view_angle = cam.getp('camva')
                projection = cam.getp('camproj')

    def _set_axis_props(self, ax):
        if DEBUG:
            print "Setting axis properties"
        self._set_title(ax)
        self._set_scale(ax)
        self._set_limits(ax)
        self._set_position(ax)
        self._set_axis_method(ax)
        self._set_daspect(ax)
        self._set_coordinate_system(ax)
        self._set_hidden_line_removal(ax)
        self._set_colorbar(ax)
        self._set_caxis(ax)
        self._set_colormap(ax)
        self._set_view(ax)
        if ax.getp('visible'):
            self._set_labels(ax)
            self._set_box(ax)
            self._set_grid(ax)
            self._g('xaxis bar on')
            self._g('xaxis tick on')
            self._g('xaxis ticklabel on')
            self._g('yaxis bar on')
            self._g('yaxis tick on')
            self._g('yaxis ticklabel on')
        else:
            # turn off all axis labeling, tickmarks, and background
            self._g('xaxis bar off')
            self._g('xaxis label ""')
            self._g('xaxis tick off')
            self._g('xaxis ticklabel off')
            self._g('yaxis bar off')
            self._g('yaxis label ""')
            self._g('yaxis tick off')
            self._g('yaxis ticklabel off')
            self._g('xaxis tick major linestyle 1')
            self._g('xaxis tick major grid off')
            self._g('yaxis tick major linestyle 1')
            self._g('yaxis tick major grid off')
            # TODO: How do we turn off axes lines?

    def _get_linespecs(self, item):
        """
        Return the line marker, line color, line style, and
        line width of the item.
        """
        
        marker = self._markers[item.getp('linemarker')]
        color = self._colors[item.getp('linecolor')]
        style = self._line_styles[item.getp('linetype')]
        width = item.getp('linewidth')
        return marker, color, style, width

    def _fix_latex(self, legend):
        """Remove latex syntax a la $, \, {, } etc."""
        legend = legend.strip()
        # General fix of latex syntax (more readable)
        legend = legend.replace('**', '^')  
        #legend = legend.replace('*', '')
        legend = legend.replace('$', '')
        legend = legend.replace('{', '')
        legend = legend.replace('}', '')
        legend = legend.replace('\\', '')
        return legend

    def _add_line(self, item, name):
        """Add a 2D or 3D curve to the scene."""
        if DEBUG:
            print "Adding a line"
        # get data:
        x = item.getp('xdata')
        y = item.getp('ydata')
        z = item.getp('zdata')
        # get line specifiactions:
        marker, color, style, width = self._get_linespecs(item)
 
        self._g('s%s on' % name)
        self._g('s%s symbol %s' % (name,marker))
        self._g('s%s symbol fill pattern 0' % name)
        self._g('s%s symbol size 0.6' % name)
        self._g('s%s symbol color %s' % (name,color))
        self._g('s%s line color %s' % (name,color))
        if style is not None:
            self._g('s%s linestyle %s' % (name,style))
        else:
            if not marker:
                self._g('s%s linestyle 1' % name)  # solid line
            else:
                self._g('s%s linestyle 0' % name)  # no line
        if not width:
            width = 1.0
        self._g('s%s linewidth %s' % (name,width))

        if z is not None:
            # zdata is given, add a 3D curve:
            pass
        else:
            # no zdata, add a 2D curve:
            for i in range(len(x)):
                self._g('s%s point %s, %s' % (name, x[i], y[i]))

        legend = self._fix_latex(item.getp('legend'))
        self._g('s%s legend "%s"' % (name,legend))
           
    def _add_surface(self, item, shading='faceted'):
        if DEBUG:
            print "Adding a surface"
        x = item.getp('xdata')  # grid component in x-direction
        y = item.getp('ydata')  # grid component in y-direction
        z = item.getp('zdata')  # scalar field
        c = item.getp('cdata')  # pseudocolor data (can be None)
        
        contours = item.getp('contours')
        if contours:
            # the current item is produced by meshc or surfc and we
            # should therefore add contours at the bottom:
            self._add_contours(contours, placement='bottom')

        if item.getp('wireframe'):
            # wireframe mesh (as produced by mesh or meshc)
            pass
        else:
            # colored surface (as produced by surf, surfc, or pcolor)
            # use keyword argument shading to set the color shading mode
            pass

    def _add_contours(self, item, placement=None):
        # The placement keyword can be either None or 'bottom'. The
        # latter specifies that the contours should be placed at the
        # bottom (as in meshc or surfc).
        if DEBUG:
            print "Adding contours"
        x = item.getp('xdata')  # grid component in x-direction
        y = item.getp('ydata')  # grid component in y-direction
        z = item.getp('zdata')  # scalar field

        filled = item.getp('filled')  # draw filled contour plot if True

        cvector = item.getp('cvector')
        clevels = item.getp('clevels')  # number of contour levels
        if cvector is None:
            # the contour levels are chosen automatically
            #cvector =
            pass

        location = item.getp('clocation')
        if location == 'surface':
            # place the contours at the corresponding z level (contour3)
            pass
        elif location == 'base':
            if placement == 'bottom':
                # place the contours at the bottom (as in meshc or surfc)
                pass
            else:
                # standard contour plot
                pass

        if item.getp('clabels'):
            # add labels on the contour curves
            pass
    
    def _add_vectors(self, item):
        if DEBUG:
            print "Adding vectors"
        # uncomment the following command if there is no support for
        # automatic scaling of vectors in the current plotting package:
        #item.scale_vectors()

        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        # vector components:
        u, v, w = item.getp('udata'), item.getp('vdata'), item.getp('wdata')
        # get line specifiactions (marker='.' means no marker):
        marker, color, style, width = self._get_linespecs(item)

        # scale the vectors according to this variable (scale=0 should
        # turn off automatic scaling):
        scale = item.getp('arrowscale')

        filled = item.getp('filledarrows') # draw filled arrows if True

        if z is not None and w is not None:
            # draw velocity vectors as arrows with components (u,v,w) at
            # points (x,y,z):
            pass
        else:
            # draw velocity vectors as arrows with components (u,v) at
            # points (x,y):
            pass

    def _add_streams(self, item):
        if DEBUG:
            print "Adding streams"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        # vector components:
        u, v, w = item.getp('udata'), item.getp('vdata'), item.getp('wdata')
        # starting positions for streams:
        sx, sy, sz = item.getp('startx'), item.getp('starty'), item.getp('startz')

        if item.getp('tubes'):
            # draw stream tubes from vector data (u,v,w) at points (x,y,z)
            n = item.getp('n') # no points along the circumference of the tube
            scale = item.getp('tubescale')
            pass
        elif item.getp('ribbons'):
            # draw stream ribbons from vector data (u,v,w) at points (x,y,z)
            width = item.getp('ribbonwidth')
            pass
        else:
            if z is not None and w is not None:
                # draw stream lines from vector data (u,v,w) at points (x,y,z)
                pass
            else:
                # draw stream lines from vector data (u,v) at points (x,y)
                pass
            pass

    def _add_isosurface(self, item):
        if DEBUG:
            print "Adding a isosurface"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        v = item.getp('vdata')  # volume
        c = item.getp('cdata')  # pseudocolor data
        isovalue = item.getp('isovalue')

    def _add_slices(self, item):
        if DEBUG:
            print "Adding slices in a volume"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        v = item.getp('vdata')  # volume

        sx, sy, sz = item.getp('slices')
        if rank(sz) == 2:
            # sx, sy, and sz defines a surface
            pass
        else:
            # sx, sy, and sz is either numbers or vectors with numbers
            pass
        pass

    def _add_contourslices(self, item):
        if DEBUG:
            print "Adding contours in slice planes"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        v = item.getp('vdata')  # volume

        sx, sy, sz = item.getp('slices')
        if rank(sz) == 2:
            # sx, sy, and sz defines a surface
            pass
        else:
            # sx, sy, and sz is either numbers or vectors with numbers
            pass

        cvector = item.getp('cvector')
        clevels = item.getp('clevels')  # number of contour levels per plane
        if cvector is None:
            # the contour levels are chosen automatically
            #cvector =
            pass
        pass

    def _set_figure_size(self, fig):
        if DEBUG:
            print "Setting figure size"
        width, height = fig.getp('size')
        if width and height:
            # set figure width and height
            self._g('page resize %s, %s' % (width,height))
        else:
            # use the default width and height in plotting package
            pass

    def figure(self, *args, **kwargs):
        # Extension of BaseClass.figure:
        # add a plotting package figure instance as fig._g and create a
        # link to it as self._g
        fig = BaseClass.figure(self, *args, **kwargs) 
        try:
            fig._g
        except:
            # create plotting package figure and save figure instance
            # as fig._g
            if DEBUG:
                name = 'Fig ' + str(fig.getp('number'))
                print "creating figure %s in backend" % name

            fig._g = grace_np.GraceProcess()
            fig._g._no_lines_in_graph = []
            
        self._g = fig._g  # link for faster access
        return fig
        
    def _replot(self):
        """Replot all axes and all plotitems in the backend."""
        # NOTE: only the current figure (gcf) is redrawn.
        if DEBUG:
            print "Doing replot in backend"
        
        fig = self.gcf()
        # add Grace attributes to current figure (if not already added):
        try:
            fig._g
        except: 
            self.figure(self.getp('curfig'))
            
        # reset the plotting package instance in fig._g:
        no_lines = fig._g._no_lines_in_graph
        no_graphs = len(no_lines)
        # erase all lines in every graph:
        for g in range(no_graphs):
            for i in range(no_lines[g]):
                fig._g('kill g%s.s%s' % (g,i))
        no_graphs = len(fig.getp('axes'))
        fig._g._no_lines_in_graph = [0]*no_graphs
        
        self._set_figure_size(fig)
        
        hgap = 0.5
        vgap = 0.6
        offset = 0.1
        nrows, ncolumns = fig.getp('axshape')
        fig._g('arrange(%s, %s, %s, %s, %s)' % \
               (nrows, ncolumns, offset, hgap, vgap))
        for axnr, ax in fig.getp('axes').items():
            curr_graph = axnr-1
            numberofitems = ax.getp('numberofitems')
            pth = ax.getp('pth')
            if pth:
                # create axes in tiled position
                # this is subplot(nrows,ncolumns,pth)
                fig._g('focus g%s' % curr_graph)
                fig._g('with g%s' % curr_graph)
                if numberofitems == 0:
                    self._g('g%s hidden true' % curr_graph)
            else:
                fig._g('with g%s' % curr_graph)
                fig._g('g%s on' % curr_graph)
            i = 0
            legends = []
            plotitems = ax.getp('plotitems')
            plotitems.sort(self._cmpPlotProperties)
            for item in plotitems:
                name = str(i)
                func = item.getp('function') # function that produced this item
                if isinstance(item, Line):
                    self._add_line(item, name)
                    fig._g._no_lines_in_graph[curr_graph] += 1
                elif isinstance(item, Surface):
                    self._add_surface(item, shading=ax.getp('shading'))
                elif isinstance(item, Contours):
                    self._add_contours(item)
                elif isinstance(item, VelocityVectors):
                    self._add_vectors(item)
                elif isinstance(item, Streams):
                    self._add_streams(item)
                elif isinstance(item, Volume):
                    if func == 'isosurface':
                        self._add_isosurface(item)
                    elif func == 'slice_':
                        self._add_slices(item)
                    elif func == 'contourslice':
                        self._add_contourslices(item)
                legend = self._fix_latex(item.getp('legend'))
                if legend:
                    # add legend to plot
                    legends.append(legend)
                i += 1
                
            if numberofitems > 0:
                self._g('autoscale')
                self._set_axis_props(ax)
                if legends:
                    self._g('legend on')

        if self.getp('show'):
            # display plot on the screen
            if DEBUG:
                print "\nDumping plot data to screen\n"
                debug(self)
            pass
        self._g('redraw')

    def hardcopy(self, filename, **kwargs):
        """
        Supported extensions in the Grace backend: 

          '.ps'  (PostScript)
          '.eps' (Encapsualted PostScript)
          '.pdf' (Portable Document Format)
          '.agr' (Grace file format)
          '.jpg' (Joint Photographic Experts Group)
          '.png' (Portable Network Graphics)
          '.pnm' (Portable Any Map)
          '.svg' (Scalable Vector Graphics)
          '.gmf' (Graphics Meta File)
          '.mif' (Maker Interchange Format)

        Optional arguments:

        ==========  ======================================================
        Argument    Description
        ==========  ======================================================
        size        A tuple (width,height) to set the size of the image.
                    The default is to use the Grace default.
        dpi         Dots per inch (Grace default is used as default).
        antialiase  Enable (True) or disable (False) font antialiasing.
                    Default is to enable antialiasing.
        color       True (colors) or False (grayscale).
        orientation 'portrait' or 'landscape' (default). Only available
                     for PostScript output.
        quality      Integer between 0 and 100 (default). Sets the quality
                     in a JPEG image.
        ==========  ======================================================

        """
        self.setp(**kwargs)
        color = self.getp('color')
        replot = kwargs.get('replot', True)
        if replot:
            self._replot()

        if DEBUG:
            print "Hardcopy to %s" % filename

        ext2dev = {'.agr': 'agr', '.eps': 'EPS', '.jpg': 'JPEG',
                   '.gmf': 'Metafile', '.mif': 'MIF', '.pdf': 'PDF',
                   '.png': 'PNG', '.pnm': 'PNM', '.ps': 'PostScript',
                   '.svg': 'SVG'}

        basename, ext = os.path.splitext(filename)
        if not ext:
            ext = '.ps'  # no extension given, assume .ps
            filename += ext
        elif ext not in ext2dev:
            raise ValueError("hardcopy: extension must be %s, not '%s'" % \
                             (ext2dev.keys(), ext))

        device = ext2dev[ext]
        if ext == '.agr':
            self._g('saveall "%s"' % filename)
        else:
            self._g('hardcopy device "%s"' % device)
            width, height = kwargs.get('size', (None,None))
            if width and height:
                self._g('device "%s" page size %s, %s' % (device,width,height))
            dpi = kwargs.get('dpi', None)
            if dpi:
                self._g('device "%s" dpi %s' % (device,dpi))
            antialiase = kwargs.get('antialiase', True)
            if antialiase:
                self._g('device "%s" font antialiasing on' % device)
            else:
                self._g('device "%s" font antialiasing off' % device)
            if device in ['EPS', 'PostScript', 'JPEG']:
                colormode = 'color'
                if not color:
                    colormode = 'grayscale'
                self._g('device "%s" op "%s"' % (device,colormode))
            if device in ['EPS', 'PostScript']:
                pass # set orientation
            if device in ['JPEG']:
                quality = kwargs.get('quality', 100)
                self._g('device "%s" op "quality:%d"' % (device,quality))
            if device in ['PNM']:
                format = 'ppm'
                if not color:
                    format = 'pgm'
                self._g('device "%s" op "format:%s"' % (device,format))
            self._g('print to "%s"' % filename)
            self._g('print')

    def _close(self, fig):
        try:
            fig._g.exit()
        except OSError, msg:
            print msg
    
    def clf(self):
        fig = gcf()
        self._close(fig)
        del fig._g
        BaseClass.clf(self)

    def closefig(self, num=None):
        if not num:  # no figure given, close current figure
            num = self._attrs['curfig']
        fig = self._figs[num]
        self._close(fig)
        del fig._g

    def closefigs(self):
        for num in self._figs.keys():
            self.closefig(num)
        BaseClass.closefigs(self)

    # implement colormap functions here
    #def jet(self, m=None):
    #    """Variant of hsv."""
    #    pass
    

    # Now we add the doc string from the methods in BaseClass to the
    # methods that are reimplemented in this backend:
    for cmd in BaseClass._matlab_like_cmds:
        if not '__' in cmd and hasattr(BaseClass, cmd):
            m1 = eval('BaseClass.%s' % cmd)
            try:
                m2 = eval('%s' % cmd)
            except NameError:
                pass
            else:
                if m1.__doc__ != m2.__doc__:
                    if m2.__doc__ is None:
                        m2.__doc__ = ""
                    m2.__doc__ = m1.__doc__ + m2.__doc__


plt = GraceBackend()  # create backend instance
use(plt, globals())   # export public namespace of plt to globals()
backend = os.path.splitext(os.path.basename(__file__))[0][:-1]



#!/usr/bin/env python

from scitools.std import *

setp(interactive=False)

x = linspace(-5,5,201)
subplot(2,2,1)
plot(x, x-sin(pi*x), xlabel='x', ylabel='y', title='subplot(2,2,1)')
subplot(2,2,2)
plot(x, x-cos(x**2), xlabel='x', ylabel='y', title='subplot(2,2,2)')
subplot(2,2,3)
plot(cos(3*pi*x), cos(0.5*pi*x), x=x, grid='on', axis=[-3,3,-3,3],
     xlabel='x', ylabel='y', title='subplot(2,2,3)')
subplot(2,2,4)
plot(x, cos(pi*x), xlabel='x', ylabel='y', title='subplot(2,2,4)')
show()

#hardcopy('subplot1a.eps', color=True)
#hardcopy('subplot1a.png', color=True)

figure()
t = linspace(0,1,51)
y1 = sin(2*pi*t)
y2 = cos(2*pi*3*t)
subplot(2,1,1)
plot(t, y1, 'ro-')
hold('on')
plot(t, y2, 'b--')
title('This is the title')
ylabel('voltage (mV)')
axis([0, 1, -1.5, 1.5])

subplot(2,1,2)
plot(t, y1+y2, 'm:')
axis([0, 1, -3, 3])
grid('on')
xlabel('time (sec)')
ylabel('voltage (mV)')
show()

#hardcopy('subplot1b.eps', color=True)
#hardcopy('subplot1b.png', color=True)

raw_input("Press Return key to quit: ")




from scitools.std import *

def f1(t):
    return t**2*exp(-t**2)

def f2(t):
    return t**2*f1(t)

t = linspace(0, 3, 51)
y1 = f1(t)
y2 = f2(t)

# Matlab-style syntax:
plot(t, y1, 'r-')
hold('on')
plot(t, y2, 'bo')

xlabel('t')
ylabel('y')
legend('t^2*exp(-t^2)', 't^4*exp(-t^2)')
title('Plotting two curves in the same plot')
savefig('tmp2.eps')

raw_input('Press Return key to quit: ')



#!/usr/bin/env python

# Example taken from:
# http://www.mathworks.com/access/helpdesk/help/techdoc/ref/contourslice.html

from scitools.easyviz import *
from time import sleep

setp(show=False)
x, y, z, v = flow()
h = contourslice(x,y,z,v,seq(1,9),[],[0],linspace(-8,2,10))
axis([0,10,-3,3,-3,3])
daspect([1,1,1])
camva(24)
camproj('perspective')
campos([-3,-15,5])
ax = gca()
ax.setp(fgcolor=(1,1,1), bgcolor=(0,0,0))
box('on')
view(3) # because camva, camproj, and campos currently not working
setp(show=True)
show()

#hardcopy('tmp_contourslice1a.eps')
#hardcopy('tmp_contourslice1a.png')

figure()
# alternative syntax:
h = contourslice(x,y,z,v,seq(1,9),[],[0],linspace(-8,2,10),
                 axis=[0,10,-3,3,-3,3], daspect=[1,1,1],
                 camva=24, camproj='perspective', campos=[-3,-15,5], 
                 fgcolor=(1,1,1), bgcolor=(0,0,0),
                 box='on')

#hardcopy('tmp_contourslice1b.eps')
#hardcopy('tmp_contourslice1b.png')

#sleep(3)
raw_input('Press Return key to quit: ')



"""
Debugging tools.

  - watch(var): print out the name, type, and value of a variable and
    where in a program this output was requested (used to monitor variables).

  - trace(message): print a message and where in the program this
    message was requested (used to trace the execution).

  - dump(obj, hide_underscore=True): print a dump of the object obj
    (attributes, methods, etc.).

  - debugregex(pattern, string): print match, groups, etc. when the
    regular expression pattern is applied to string.

watch and trace prints information only if the module variable
DEBUG has a true value. DEBUG can be initialized from an environment
variable PYDEBUG, otherwise it is set to 1 by default. Other
modules can monitor their debugging by setting debug.DEBUG = 0
or debug.DEBUG = 1 (note that a single such setting has a "global" effect;
it turns off debugging everywhere).
"""
import os, sys, string, re, inspect

__all__ = ['watch', 'trace', 'dump', 'debugregex']

if 'PYDEBUG' in os.environ:
    DEBUG = int(os.environ['PYDEBUG'])
else:
    # import user?
    DEBUG = 1

if not __debug__:  # python -O
    DEBUG = 0      # turn off debugging
    
import traceback
import errorcheck

def watch(variable, output_medium=sys.stdout):
    """
    Print out the name, type, and value of a variable and
    where in a program this output was requested.
    Used to monitor variables during debugging.
    As an example, watch(myprm) may lead to this output::

      myprm <int> in ./myscript.py(56): myfunction
        = 3

    The variable is returned to the caller, thus it can be used
    inside expressions:
      myobj.setCallback(lambda self: self.setstate(watch(self.mystate)))
    or
      dosomething(*watch((arg1, arg2, arg3)))

    (This function is a modified version of a function taken
    from the online Python Cookbook::

      http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/52314/index_txt
      
    The original code was written by Olivier Dagenais).
    """
    if not DEBUG:
        return variable
    stack = traceback.extract_stack()[-2:][0]
    actual_call = stack[3]
    if actual_call is None:
        actual_call = "watch([unknown])"
    left = string.find(actual_call, '(' )
    right = string.rfind(actual_call, ')')
    prm = {}
    # variable name is extracted from actual_call:
    # everything between '(' and ')'
    prm["variable_name"] = string.strip(actual_call[left+1:right])  
    prm["variable_type"] = errorcheck.get_type(variable)  # str(type(variable))[7:-2]
    if isinstance(variable, basestring):
        print_variable = str(variable)
    else:
        print_variable = repr(variable)
    prm["value"]       = print_variable
    prm["method_name"]  = stack[2]
    prm["line_number"] = stack[1]
    prm["filename"]    = stack[0]
    prm["shortfile"]   = shorten_path(stack[0])
    if prm["method_name"] == '?':
        prm["method_name"] = 'main'
        #outstr = 'watch: %(shortfile)s(%(line_number)d): '\
        #     '%(method_name)s\n  %(variable_name)s '\
        #     '<%(variable_type)s> = %(value)s\n'
    outstr = '%(variable_name)s <%(variable_type)s>'\
             ' in %(shortfile)s(%(line_number)d): %(method_name)s\n'\
             '  = %(value)s\n'
    output_medium.write(outstr % prm)
    return variable

def shorten_path(path, abb_len=20):
    """Shorten a path to relative (./rel_path) or abbreviated (...rest_of_path)
    form. abb_len=0 for no abbreviation of the second form."""
    cwd = os.environ.get('PWD') or os.getcwd()
    if path.startswith(cwd):
        return "."+path[len(cwd):]
    else:
        return path[-abb_len:]


def trace(message='', output_medium=sys.stdout, frameno=-2):
    """
    Print a message and where in the program this
    message was requested (as in the function watch).
    Used to trace the program flow during debugging.

    If called from constructors, and sometimes also other class
    methods with "generic" names, it may be smart to let the message
    be the classname::

      class A:
        def __init__(self):
          debug.trace(self.__class__.__name__)

        def write(self):
          debug.trace(self.__class__.__name__)

    With frameno=-2 the place where this debug.trace function is called is
    printed, while smaller values gives printout of previous calls on the call
    stack. In the previous example, we could look further back to say:

      class A:
        def __init__(self):
            debug.trace('Instantiation of '+self.__class__.__name__, frameno=-3)

        def write(self):
            debug.trace('A.write() call', frameno=-3)

    Setting frameno=-4 may also be useful in some particular cases, for example
    in a baseclass __init__ which is usually overridden. But there is danger
    that we then look too far back in the call stack.

    (This function is a modified version of one taken from the
     Python Cookbook, see the watch function.)
  """
    if not DEBUG:
        return
    stack = traceback.extract_stack()[frameno:][0]
    prm = {}
    prm["method_name"] = stack[2]
    prm["line_number"] = stack[1]
    prm["filename"]    = stack[0]
    prm["shortfile"]   = shorten_path(stack[0])
    prm["message"]     = message
    outstr = '%(shortfile)s(%(line_number)d) %(method_name)s: %(message)s\n'
    output_medium.write(outstr % prm)

def dump(obj, hide_nonpublic=True):
    """
    Inspect an object obj by the dir function.
    This dump function first prints the repr(obj) of the object,
    then all data attributes and their values are
    listed, and finally a line listing all functions/methods
    is printed.
    """
    print '\n', '*'*60, '\n'
    try:  # dump class name if it exists
        print 'object of class', obj.__class__.__name__
    except:
        pass
    try:
        print 'object with name %s' % obj.__name__
    except:
        pass
    methods = [];  attrs = []
    for a in dir(obj):
        if a.startswith('_') and hide_nonpublic:
            pass
        else:
            try:
                attr = getattr(obj, a)
                s = a + '=' + str(attr)
                if s.find(' method ') != -1 or \
                   s.find('function ') != -1 or \
                   s.find('method-wrapper ') != -1 or \
                   s.find('ufunc ') != -1:
                    try:
                        # add argument list (args, varargs, kwargs, defaults)
                        a = '%-15s%s' % (a, inspect.formatargspec(inspect.getargspec(attr)))
                    except:
                        pass
                    methods.append(a) # skip function addresses
                else:
                    s += '  (' + errorcheck.get_type(attr) + ')'
                    attrs.append(s)  # variable=value
            except Exception, msg:
                pass
    print '******** data attributes:\n', '\n'.join(attrs)
    print '\n******** methods:\n', '\n'.join(methods)
    print '*'*60, '\n\n\n', 


def debugregex(pattern, string):
    "debugging of regular expressions: write the match and the groups"
    s = "does '" + pattern + "' match '" + string + "'?\n"
    match = re.search(pattern, string)
    if match:
        s += string[:match.start()] + '[' + \
             string[match.start():match.end()] + \
             ']' + string[match.end():]
        if len(match.groups()) > 0:
            for i in range(len(match.groups())):
                s += '\ngroup %d: [%s]' % (i+1,match.groups()[i])
    else:
        s += 'No match'
    return s

def setprofile(include='', exclude=None, output=sys.stdout):
    """
    Print a message on method call/return/exception.

    @param include: A regular expression for output filtering. The regular
    expression is applied to the output string, which is on the format 'event
    filename(line): classname.methodname'.
    
    For example, setprofile(os.getcwd()) prints only methods that are defined
    in python files in this directory and subdirectories. setprofile('^c_')
    prints only C invocations. And so on.

    @param exclude: Another regular expression.
    """
    include = re.compile(include)
    exclude = exclude and re.compile(exclude)
    def prof(frame, event, arg):
        meth = frame.f_code.co_name
        try:
            cls = frame.f_locals['self']
            if isinstance(cls, object):
                cls = cls.__class__
            meth = '.'.join([cls.__name__, meth])
        except KeyError:
            pass
        s = '%s: %s(%s): %s\n' % (event, frame.f_code.co_filename,
                                  frame.f_lineno, meth)
        if include.search(s) and not (exclude and exclude.search(s)):
            output.write(s)
    sys.setprofile(prof)



"""Test functions for colormap command."""

from easyviztest import *


class test_colormap_basic(EasyvizTestCase):
    def check_colormap_default(self):
        x,y,xv,yv,values = self.get_2D_data()
        pcolor(xv,yv,values)
        colormap('default')
        title("pcolor(..);colormap('default')")
        n()

    def check_colormap_default_kwarg(self):
        x,y,xv,yv,values = self.get_2D_data()
        pcolor(xv,yv,values,colormap='default')
        title("pcolor(..,colormap='default')")
        n()

    def check_colormap_jet(self):
        x,y,xv,yv,values = self.get_2D_data()
        pcolor(xv,yv,values)
        colormap(jet())
        title("pcolor(..);colormap(jet())")
        n()

##     def check_colormap_jet_string(self):
##         x,y,xv,yv,values = self.get_2D_data()
##         pcolor(xv,yv,values)
##         colormap('jet')
##         title("colormap('jet')")
##         n()


class test_colormap_jet(EasyvizTestCase):
    def check_colormap_jet_4_colors(self):
        x,y,xv,yv,values = self.get_2D_data(n=31)
        pcolor(xv,yv,values)
        colormap(jet(4))
        title("pcolor(..);colormap(jet(4))")
        n()
        
    def check_colormap_jet_16_colors(self):
        x,y,xv,yv,values = self.get_2D_data(n=31)
        pcolor(xv,yv,values)
        colormap(jet(16))
        title("pcolor(..);colormap(jet(16))")
        n()

    def check_colormap_jet_128_colors(self):
        x,y,xv,yv,values = self.get_2D_data(n=31)
        pcolor(xv,yv,values)
        colormap(jet(128))
        title("pcolor(..);colormap(jet(128))")
        n()


class test_colormap_hsv(EasyvizTestCase):
    def check_colormap_hsv_4_colors(self):
        x,y,xv,yv,values = self.get_2D_data(n=31)
        pcolor(xv,yv,values,colormap=hsv(4))
        title("pcolor(..,colormap=hsv(4))")
        n()

    def check_colormap_hsv_16_colors(self):
        x,y,xv,yv,values = self.get_2D_data(n=31)
        pcolor(xv,yv,values,colormap=hsv(16))
        title("pcolor(..,colormap=hsv(16))")
        n()

    def check_colormap_hsv_128_colors(self):
        x,y,xv,yv,values = self.get_2D_data(n=31)
        pcolor(xv,yv,values,colormap=hsv(128))
        title("pcolor(..,colormap=hsv(128))")
        n()

class test_colormap_hot(EasyvizTestCase):
    def check_colormap_hot_4_colors(self):
        x,y,xv,yv,values = self.get_2D_data(n=31)
        pcolor(xv,yv,values,colormap=hot(4))
        title("pcolor(..,colormap=hot(4))")
        n()

    def check_colormap_hot_16_colors(self):
        x,y,xv,yv,values = self.get_2D_data(n=31)
        pcolor(xv,yv,values,colormap=hot(16))
        title("pcolor(..,colormap=hot(16))")
        n()

    def check_colormap_hot_128_colors(self):
        x,y,xv,yv,values = self.get_2D_data(n=31)
        pcolor(xv,yv,values,colormap=hot(128))
        title("pcolor(..,colormap=hot(128))")
        n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_colormap_basic,'check_'))
        suites.append(unittest.makeSuite(test_colormap_jet,'check_'))
        suites.append(unittest.makeSuite(test_colormap_hsv,'check_'))
        suites.append(unittest.makeSuite(test_colormap_hot,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")



"""Test functions for x-, y-, and zlabel commands."""

from easyviztest import *


class test_xlabel_simple(EasyvizTestCase):
    def check_xlabel(self):
        x,y,v,w = self.get_line_data()
        plot(x,y)
        xlabel('XLABEL')
        title("plot(..);xlabel('XLABEL')")
        n()

    def check_xlabel_kwarg(self):
        x,y,v,w = self.get_line_data()
        plot(x,y,xlabel='XLABEL')
        title("plot(..,xlabel='XLABEL')")
        n()


class test_ylabel_simple(EasyvizTestCase):
    def check_ylabel(self):
        x,y,v,w = self.get_line_data()
        plot(x,y)
        ylabel('YLABEL')
        title("plot(..);ylabel('YLABEL')")
        n()

    def check_ylabel_kwarg(self):
        x,y,v,w = self.get_line_data()
        plot(x,y,ylabel='YLABEL')
        title("plot(..,ylabel='YLABEL')")
        n()


class test_zlabel_simple(EasyvizTestCase):
    def check_zlabel(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        zlabel('ZLABEL')
        title("surf(..);zlabel('ZLABEL')")
        n()

    def check_zlabel_kwarg(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values,zlabel='ZLABEL')
        title("surf(..,zlabel='ZLABEL')")
        n()


class test_xylabels_combined(EasyvizTestCase):
    def check_xylabel(self):
        x,y,v,w = self.get_line_data()
        plot(x,y)
        xlabel('XLABEL')
        ylabel('YLABEL')
        title("plot(..);xlabel('XLABEL');ylabel('YLABEL')")
        n()

    def check_xylabel_kwargs(self):
        x,y,v,w = self.get_line_data()
        plot(x,y,xlabel='XLABEL',ylabel='YLABEL')
        title("plot(..,xlabel='XLABEL',ylabel='YLABEL')")
        n()


class test_xyzlabels_combined(EasyvizTestCase):
    def check_xyzlabel(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        xlabel('XLABEL')
        ylabel('YLABEL')
        zlabel('ZLABEL')
        title("surf(..);xlabel('XLABEL');ylabel(YLABEL');zlabel('ZLABEL')")
        n()

    def check_xyzlabel_kwargs(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values,xlabel='XLABEL',ylabel='YLABEL',zlabel='ZLABEL')
        title("surf(..,xlabel='XLABEL',ylabel='YLABEL',zlabel='ZLABEL')")
        n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_xlabel_simple,'check_'))
        suites.append(unittest.makeSuite(test_ylabel_simple,'check_'))
        suites.append(unittest.makeSuite(test_zlabel_simple,'check_'))
        suites.append(unittest.makeSuite(test_xylabels_combined,'check_'))
        suites.append(unittest.makeSuite(test_xyzlabels_combined,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")



#!/usr/bin/env python

# Example taken from:
# http://www.mathworks.com/access/helpdesk/help/techdoc/visualize/f5-3796.html

from scitools.easyviz import *
from time import sleep
from scipy import io

wind = io.loadmat('wind_matlab_v6.mat')
x = wind['x']
y = wind['y']
z = wind['z']
u = wind['u']
v = wind['v']
w = wind['w']

# Determine the Range of the Coordinates:
xmin = arrmin(x)
xmax = arrmax(x)
ymax = arrmax(y)
zmin = arrmin(z)
ymax = ymax - 0.1
zmin = 0 # bug in slice_ and contourslice

setp(interactive=False)

# Add Slice Planes for Visual Context:
wind_speed = sqrt(u**2 + v**2 + w**2)
hsurfaces = slice_(x,y,z,wind_speed,[xmin,100,xmax],ymax,zmin)
#set(hsurfaces,'FaceColor','interp','EdgeColor','none')
hold('on')
shading('interp')

# Add Contour Lines to Slice Planes:
hcont = contourslice(x,y,z,wind_speed,[xmin,100,xmax],ymax,zmin,8)
# What is the default number of contour lines in contourslice? 8?
#set(hcont,'EdgeColor',[.7,.7,.7],'LineWidth',.5)
hcont.setp(linecolor=[.7,.7,.7], linewidth=2)

# Define the Starting Points for the Stream Lines:
sx,sy,sz = ndgrid([80]*4,seq(20,50,10),seq(0,15,5), sparse=False)
hlines = streamline(x,y,z,u,v,w,sx,sy,sz)
#set(hlines,'LineWidth',2,'Color','r')
hlines.setp(linewidth=3, linecolor='r')

# Define the View:
view(3)
daspect([2,2,1])
axis('tight')

show()

raw_input('Press Return key to quit: ')



import math, numpy, wave, commands, sys, os

max_amplitude = 2**15-1 # iinfo('int16').max if numpy >= 1.0.3

def write(data, filename, channels=1, sample_width=2, sample_rate=44100):
    """
    Writes the array data to the specified file.
    The array data type can be arbitrary as it will be
    converted to numpy.int16 in this function.
    """
    ofile = wave.open(filename, 'w')
    ofile.setnchannels(channels)
    ofile.setsampwidth(sample_width)
    ofile.setframerate(sample_rate)
    ofile.writeframesraw(data.astype(numpy.int16).tostring())
    ofile.close()

def read(filename):
    """
    Read sound data in a file and return the data as an array
    with data type numpy.int16.
    """
    ifile = wave.open(filename)
    channels = ifile.getnchannels()
    sample_width = ifile.getsampwidth()
    sample_rate = ifile.getframerate()
    frames = ifile.getnframes()
    data = ifile.readframes(frames)
    data = numpy.fromstring(data, dtype=numpy.uint16)
    return data.astype(numpy.int16)

def play(soundfile, player=None):
    """
    Play a file with name soundfile or an array soundfile.
    (The array is first written to file using the write function
    so the data type can be arbitrary.)
    The player is chosen by the programs 'open' on Mac and 'start'
    on Windows. On Linux, try different open programs for various
    distributions. If keyword argument 'player' is given, only this spesific
    commands is run.
    """
    tmpfile = 'tmp.wav'
    if isinstance(soundfile, numpy.ndarray):
        write(soundfile, tmpfile)
    elif isinstance(soundfile, str):
        tmpfile = soundfile

    if player:
        msg = 'Unable to open sound file with %s' %player
        if sys.platform[:3] == 'win':
            status = os.system('%s %s' %(player, tmpfile))
        else:
            status, output = commands.getstatusoutput('%s %s' %(player, tmpfile))
            msg += '\nError message:\n%s' %output
        if status != 0:
            raise OSError(msg)
        return

    if sys.platform[:5] == 'linux':
        open_commands = ['gnome-open', 'kmfclient exec', 'exo-open', 'xdg-open', 'open']
        for cmd in open_commands:
            status, output = commands.getstatusoutput('%s %s' %(cmd, tmpfile))
            if status == 0:
                break
        if status != 0:
            raise OSError('Unable to open sound file, try to set player'\
                              ' keyword argument.')

    elif sys.platform == 'darwin':
        commands.getstatusoutput('open %s' %tmpfile)
    else:
        # assume windows
        os.system('start %s' %tmpfile)

def note(frequency, length, amplitude=1, sample_rate=44100):
    """
    Generate the sound of a note as an array if float elements.
    """
    time_points = numpy.linspace(0, length, length*sample_rate)
    data = numpy.sin(2*numpy.pi*frequency*time_points)
    data = amplitude*data
    return data

def add_echo(data, beta=0.8, delay=0.1, sample_rate=44100):
    newdata = data.copy()
    shift = int(delay*sample_rate)  # b (math symbol)
    newdata[shift:] = beta*data[shift:] + \
                      (1-beta)*data[:len(data)-shift]
    #for i in xrange(shift, len(data)):
    #    newdata[i] = beta*data[i] + (1-beta)*data[i-shift]
    return newdata

def _test1():
    filename = 'tmp.wav'

    tone1 = max_amplitude*note(440, 1, 0.2)
    tone2 = max_amplitude*note(293.66, 1, 1)
    tone3 = max_amplitude*note(440, 1, 0.8)
    data = numpy.concatenate((tone1, tone2, tone3))
    write(data, filename)
    data = read(filename)
    play(filename)

def Nothing_Else_Matters(echo=False):
    E1 = note(164.81, .5)
    G = note(392, .5)
    B = note(493.88, .5)
    E2 = note(659.26, .5)
    intro = numpy.concatenate((E1, G, B, E2, B, G))
    high1_long = note(987.77, 1)
    high1_short = note(987.77, .5)
    high2 = note(1046.50, .5)
    high3 = note(880, .5)
    high4_long = note(659.26, 1)
    high4_medium = note(659.26, .5)
    high4_short = note(659.26, .25)
    high5 = note(739.99, .25)
    pause_long =  note(0, .5)
    pause_short = note(0, .25)
    song = numpy.concatenate(
      (intro, intro, high1_long, pause_long, high1_long,
       pause_long, pause_long,
       high1_short, high2, high1_short, high3, high1_short,
       high3, high4_short, pause_short, high4_long, pause_short,
       high4_medium, high5, high4_short))
    song *= max_amplitude
    if echo:
        song = add_echo(song, 0.6)
    return song

# equal-tempered scale:
note2freq = {
 'A#0': 29.14,
 'A#1': 58.27,
 'A#2': 116.54,
 'A#3': 233.08,
 'A#4': 466.16,
 'A#5': 932.33,
 'A#6': 1864.66,
 'A#7': 3729.31,
 'A0': 27.5,
 'A1': 55,
 'A2': 110,
 'A3': 220,
 'A4': 440,
 'A5': 880,
 'A6': 1760,
 'A7': 3520,
 'Ab0': 25.96,
 'Ab1': 51.91,
 'Ab2': 103.83,
 'Ab3': 207.65,
 'Ab4': 415.3,
 'Ab5': 830.61,
 'Ab6': 1661.22,
 'Ab7': 3322.44,
 'B0': 30.87,
 'B1': 61.74,
 'B2': 123.47,
 'B3': 246.94,
 'B4': 493.88,
 'B5': 987.77,
 'B6': 1975.53,
 'B7': 3951.07,
 'Bb0': 29.14,
 'Bb1': 58.27,
 'Bb2': 116.54,
 'Bb3': 233.08,
 'Bb4': 466.16,
 'Bb5': 932.33,
 'Bb6': 1864.66,
 'Bb7': 3729.31,
 'C#0': 17.32,
 'C#1': 34.65,
 'C#2': 69.3,
 'C#3': 138.59,
 'C#4': 277.18,
 'C#5': 554.37,
 'C#6': 1108.73,
 'C#7': 2217.46,
 'C#8': 4434.92,
 'C1': 32.7,
 'C2': 65.41,
 'C3': 130.81,
 'C4': 261.63,
 'C5': 523.25,
 'C6': 1046.5,
 'C7': 2093,
 'C8': 4186.01,
 'D#0': 19.45,
 'D#1': 38.89,
 'D#2': 77.78,
 'D#3': 155.56,
 'D#4': 311.13,
 'D#5': 622.25,
 'D#6': 1244.51,
 'D#7': 2489.02,
 'D#8': 4978.03,
 'D0': 18.35,
 'D1': 36.71,
 'D2': 73.42,
 'D3': 146.83,
 'D4': 293.66,
 'D5': 587.33,
 'D6': 1174.66,
 'D7': 2349.32,
 'D8': 4698.64,
 'Db0': 17.32,
 'Db1': 34.65,
 'Db2': 69.3,
 'Db3': 138.59,
 'Db4': 277.18,
 'Db5': 554.37,
 'Db6': 1108.73,
 'Db7': 2217.46,
 'Db8': 4434.92,
 'E0': 20.6,
 'E1': 41.2,
 'E2': 82.41,
 'E3': 164.81,
 'E4': 329.63,
 'E5': 659.26,
 'E6': 1318.51,
 'E7': 2637.02,
 'Eb0': 19.45,
 'Eb1': 38.89,
 'Eb2': 77.78,
 'Eb3': 155.56,
 'Eb4': 311.13,
 'Eb5': 622.25,
 'Eb6': 1244.51,
 'Eb7': 2489.02,
 'Eb8': 4978.03,
 'F#0': 23.12,
 'F#1': 46.25,
 'F#2': 92.5,
 'F#3': 185,
 'F#4': 369.99,
 'F#5': 739.99,
 'F#6': 1479.98,
 'F#7': 2959.96,
 'F0': 21.83,
 'F1': 43.65,
 'F2': 87.31,
 'F3': 174.61,
 'F4': 349.23,
 'F5': 698.46,
 'F6': 1396.91,
 'F7': 2793.83,
 'G#0': 25.96,
 'G#1': 51.91,
 'G#2': 103.83,
 'G#3': 207.65,
 'G#4': 415.3,
 'G#5': 830.61,
 'G#6': 1661.22,
 'G#7': 3322.44,
 'G0': 24.5,
 'G1': 49,
 'G2': 98,
 'G3': 196,
 'G4': 392,
 'G5': 783.99,
 'G6': 1567.98,
 'G7': 3135.96,
 'Gb0': 23.12,
 'Gb1': 46.25,
 'Gb2': 92.5,
 'Gb3': 185,
 'Gb4': 369.99,
 'Gb5': 739.99,
 'Gb6': 1479.98,
 'Gb7': 2959.96}
    
if __name__ == '__main__':
    #_test1()
    song = Nothing_Else_Matters(False)
    play(song)



"""
This file contains physical properties of various materials,
organized as a dictionary of dictionaries. The keys in the
outer dict are the materials, while the keys in the inner
dicts correspond to physical properties.
"""

materials = {

"Acetone [C3H6O] (20 C)": {
   "Melting Point (C)":                     -94.85,
   "Viscosity (Pa*s)":                      0.000389,
   "Thermal Conductivity (W/m*K)":          0.161,
   "Density (kg/m^3)":                      0.7899E+3,
   "Boiling Point (C)":                     56.05,
},
"Acetone [C3H6O] (25 C)": {
   "Sound Velocity (m/s)":                  1174.0,
   "-dV/dt (m/s*C)":                        4.5,
},
"Air (0 C, dry)": {
   "Boiling Point (C)":                     -195.0,
   "Density (kg/m^3)":                      0.00129E+3,
   "-dV/dt (m/s*C)":                        -0.59,
   "Kinematic Viscosity (m2/s)":            1.33e-05,
   "Viscosity (Pa*s)":                      1.71e-05,
   "Sound Velocity (m/s)":                  331.4,
},
"Air (25 C, dry)": {
   "Kinematic Viscosity (m2/s)":            1.46e-05,
   "Viscosity (Pa*s)":                      1.79e-05,
   "Sound Velocity (m/s)":                  346.3,
   "Density (kg/m^3)":                      0.001184E+3,
   "-dV/dt (m/s*C)":                        -0.59,
},
"Alcohol; Ethanol [C2H5OH] (20 C)": {
   "Boiling Point (C)":                     78.29,
   "Viscosity (Pa*s)":                      0.00177,
   "Density (kg/m^3)":                      0.789E+3,
   "Kinematic Viscosity (m2/s)":            2.2e-06,
   "Melting Point (C)":                     -114.2,
   "Bulk Modulus (GPa)":                    0.823,
   "Thermal Conductivity (W/m*K)":          0.169,
},
"Alcohol; Ethanol [C2H5OH] (25 C)": {
   "Sound Velocity (m/s)":                  1207.0,
   "-dV/dt (m/s*C)":                        4.0,
},
"Alcohol; Methanol [CH3OH] (20 C)": {
   "Boiling Point (C)":                     64.55,
   "Viscosity (Pa*s)":                      0.000817,
   "Density (kg/m^3)":                      0.792E+3,
   "Kinematic Viscosity (m2/s)":            1.01e-06,
   "Melting Point (C)":                     -97.68,
   "Bulk Modulus (GPa)":                    0.902,
   "Thermal Conductivity (W/m*K)":          0.2,
},
"Alcohol; Methanol [CH3OH] (25 C)": {
   "Sound Velocity (m/s)":                  1103.0,
   "-dV/dt (m/s*C)":                        3.2,
},
"Aluminum Alloy": {
   "Electric Resistivity (W*m)":            50.0E-9,
   "Ultimate Stress (MPa)":                 [100.0, 550.0],
   "Poisson's Ratio":                       0.33,
   "Shear Modulus (GPa)":                   [26.0, 30.0],
   "Density (kg/m^3)":                      [2.64E+3, 2.8E+3],
   "Melting Point (C)":                     [565.0, 660.0],
   "Thermal Expansion Coefficient (1/C)":   23.0E-6,
   "Elongation (%)":                        [1.0, 45.0],
   "Elastic Modulus (GPa)":                 [70.0, 79.0],
   "Yield Stress (MPa)":                    [35.0, 500.0],
},
"Aluminum Alloy 6082": {
   "Density (kg/m^3)":                      2.7E+3,
   "Thermal Expansion Coefficient (1/C)":   2.5E-5,
   "Thermal Conductivity (W/m*K)":          200.0,
   "Specific Heat (J/kg*C)":                900,
},
"Aluminum [Al]": {
   "Boiling Point (C)":                     2519.0,
   "Electric Resistivity (W*m)":            27.0E-9,
   "Ultimate Stress (MPa)":                 70.0,
   "Poisson's Ratio":                       0.33,
   "Shear Modulus (GPa)":                   26.0,
   "Density (kg/m^3)":                      2.71E+3,
   "Melting Point (C)":                     660.3,
   "Thermal Expansion Coefficient (1/C)":   23.0E-6,
   "Elongation (%)":                        60.0,
   "Thermal Conductivity (W/m*K)":          237.0,
   "Elastic Modulus (GPa)":                 70.0,
   "Yield Stress (MPa)":                    20.0,
},
"Aluminum [Al] (Rolled)": {
   "Shear Wave Velocity (m/s)":             3040.0,
   "Longitudinal Wave Velocity (m/s)":      6420.0,
   "Bar Velocity (m/s)":                    5000.0,
},
"Ammonia [H3N] (0 C)": {
   "Sound Velocity (m/s)":                  415.0,
},
"Argon [Ar] (0 C)": {
   "Melting Point (C)":                     -189.3,
   "Density (kg/m^3)":                      0.001784E+3,
   "Boiling Point (C)":                     -185.8,
},
"Argon [Ar] (20 C)": {
   "Sound Velocity (m/s)":                  319.0,
   "-dV/dt (m/s*C)":                        -0.56,
},
"Brass": {
   "Longitudinal Wave Velocity (m/s)":      4700.0,
   "Electric Resistivity (W*m)":            [20.0E-9, 61.0E-9],
   "Ultimate Stress (MPa)":                 [200.0, 620.0],
   "Poisson's Ratio":                       0.34,
   "Shear Modulus (GPa)":                   [36.0, 41.0],
   "Density (kg/m^3)":                      [8.4E+3, 8.75E+3],
   "Melting Point (C)":                     930.0,
   "Shear Wave Velocity (m/s)":             2110.0,
   "Thermal Expansion Coefficient (1/C)":   [19.1E-6, 21.2E-6],
   "Elongation (%)":                        [4.0, 60.0],
   "Elastic Modulus (GPa)":                 [96.0, 110.0],
   "Bar Velocity (m/s)":                    3480.0,
   "Yield Stress (MPa)":                    [70.0, 550.0],
},
"Brass; Noval": {
   "Ultimate Stress (MPa)":                 [410.0, 590.0],
   "Poisson's Ratio":                       0.34,
   "Shear Modulus (GPa)":                   39.0,
   "Density (kg/m^3)":                      8.4E+3,
   "Thermal Expansion Coefficient (1/C)":   21.1E-6,
   "Elongation (%)":                        [15.0, 50.0],
   "Elastic Modulus (GPa)":                 100.0,
   "Yield Stress (MPa)":                    [170.0, 410.0],
},
"Brass; Red (80% Cu, 20% Zn)": {
   "Ultimate Stress (MPa)":                 [300.0, 590.0],
   "Poisson's Ratio":                       0.34,
   "Shear Modulus (GPa)":                   39.0,
   "Density (kg/m^3)":                      8.75E+3,
   "Melting Point (C)":                     1000.0,
   "Thermal Expansion Coefficient (1/C)":   19.1E-6,
   "Elongation (%)":                        [4.0, 50.0],
   "Elastic Modulus (GPa)":                 100.0,
   "Yield Stress (MPa)":                    [90.0, 470.0],
},
"Brick": {
   "Bar Velocity (m/s)":                    3650.0,
   "Thermal Expansion Coefficient (1/C)":   [5.0E-6, 7.0E-6],
   "Density (kg/m^3)":                      [1.8E+3, 2.4E+3],
   "Ultimate Stress (MPa)":                 [7.0, 70.0],
},
"Brick (Compression)": {
   "Elastic Modulus (GPa)":                 [10.0, 24.0],
},
"Bronze; Manganese": {
   "Ultimate Stress (MPa)":                 [450.0, 620.0],
   "Poisson's Ratio":                       0.34,
   "Shear Modulus (GPa)":                   39.0,
   "Density (kg/m^3)":                      8.3E+3,
   "Thermal Expansion Coefficient (1/C)":   20.0E-6,
   "Elongation (%)":                        [10.0, 35.0],
   "Elastic Modulus (GPa)":                 100.0,
   "Yield Stress (MPa)":                    [170.0, 450.0],
},
"Bronze; Regular": {
   "Ultimate Stress (MPa)":                 [200.0, 830.0],
   "Poisson's Ratio":                       0.34,
   "Shear Modulus (GPa)":                   [36.0, 44.0],
   "Density (kg/m^3)":                      [7.8E+3, 8.8E+3],
   "Melting Point (C)":                     1050.0,
   "Thermal Expansion Coefficient (1/C)":   [18.0E-6, 21.0E-6],
   "Elongation (%)":                        [5.0, 60.0],
   "Elastic Modulus (GPa)":                 [96.0, 120.0],
   "Yield Stress (MPa)":                    [82.0, 690.0],
},
"Carbon Dioxide [CO2] (0 C)": {
   "Melting Point (C)":                     -56.57,
   "Viscosity (Pa*s)":                      1.38e-05,
},
"Carbon Dioxide [CO2] (25 C)": {
   "Sound Velocity (m/s)":                  259.0,
   "-dV/dt (m/s*C)":                        -0.4,
},
"Carbon Monoxide [CO] (25 C)": {
   "Sound Velocity (m/s)":                  338.0,
   "-dV/dt (m/s*C)":                        -0.6,
},
"Carbon [C]": {
   "Melting Point (C)":                     4492.0,
   "Elastic Modulus (GPa)":                 6.9,
   "Electric Resistivity (W*m)":            14100.0E-9,
   "Density (kg/m^3)":                      2.25E+3,
   "Boiling Point (C)":                     3642.0,
},
"Ceramic": {
   "Melting Point (C)":                     3870.0,
   "Elastic Modulus (GPa)":                 [300.0, 400.0],
   "Density (kg/m^3)":                      [2.0E+3, 3.0E+3],
},
"Concrete": {
   "Poisson's Ratio":                       [0.1, 0.2],
   "Elastic Modulus (GPa)":                 [18.0, 30.0],
   "Density (kg/m^3)":                      [2.3E+3, 2.4E+3],
   "Thermal Expansion Coefficient (1/C)":   [7.0E-6, 14.0E-6],
},
"Concrete (Compression)": {
   "Ultimate Stress (MPa)":                 [10.0, 70.0],
},
"Copper Alloy": {
   "Electric Resistivity (W*m)":            [17.0E-9, 490.0E-9],
   "Ultimate Stress (MPa)":                 830.0,
   "Shear Modulus (GPa)":                   47.0,
   "Density (kg/m^3)":                      8.23E+3,
   "Melting Point (C)":                     925.0,
   "Thermal Expansion Coefficient (1/C)":   17.0E-6,
   "Elongation (%)":                        4.0,
   "Elastic Modulus (GPa)":                 120.0,
   "Yield Stress (MPa)":                    760.0,
},
"Copper [Cu]": {
   "Boiling Point (C)":                     2562.0,
   "Electric Resistivity (W*m)":            17.0E-9,
   "Ultimate Stress (MPa)":                 [230.0, 380.0],
   "Poisson's Ratio":                       [0.33, 0.36],
   "Shear Modulus (GPa)":                   [40.0, 47.0],
   "Density (kg/m^3)":                      8.94E+3,
   "Melting Point (C)":                     1085.0,
   "Thermal Expansion Coefficient (1/C)":   [16.6E-6, 17.6E-6],
   "Elongation (%)":                        [10.0, 50.0],
   "Thermal Conductivity (W/m*K)":          410.0,
   "Elastic Modulus (GPa)":                 [110.0, 120.0],
   "Yield Stress (MPa)":                    [55.0, 330.0],
},
"Copper [Cu] (Annealed/Rolled)": {
   "Shear Wave Velocity (m/s)":              2325 / 2270,
   "Longitudinal Wave Velocity (m/s)":       4760 / 5010,
   "Bar Velocity (m/s)":                     3810 / 3750,
},
"Cork": {
   "Poisson's Ratio":                       0.0,
   "Bar Velocity (m/s)":                    500.0,
   "Density (kg/m^3)":                      [0.15E+3, 0.2E+3],
},
"Glass": {
   "Longitudinal Wave Velocity (m/s)":      [3980.0, 5640.0],
   "Ultimate Stress (MPa)":                 [30.0, 1000.0],
   "Poisson's Ratio":                       [0.2, 0.27],
   "Shear Modulus (GPa)":                   [19.0, 34.0],
   "Density (kg/m^3)":                      [2.4E+3, 2.8E+3],
   "Shear Wave Velocity (m/s)":             [2380.0, 3280.0],
   "Thermal Expansion Coefficient (1/C)":   [5.0E-6, 11.0E-6],
   "Bar Velocity (m/s)":                    [3720.0, 5170.0],
   "Elastic Modulus (GPa)":                 [48.0, 83.0],
},
"Gold [Au]": {
   "Boiling Point (C)":                     2856.0,
   "Electric Resistivity (W*m)":            24.0E-9,
   "Poisson's Ratio":                       0.44,
   "Density (kg/m^3)":                      19.32E+3,
   "Melting Point (C)":                     1064.0,
   "Thermal Conductivity (W/m*K)":          317.0,
   "Elastic Modulus (GPa)":                 83.0,
},
"Gold [Au] (Hard-drawn)": {
   "Shear Wave Velocity (m/s)":             1200.0,
   "Longitudinal Wave Velocity (m/s)":      3240.0,
   "Bar Velocity (m/s)":                    2030.0,
},
"Helium [He] (0 C)": {
   "Boiling Point (C)":                     -268.9,
   "Density (kg/m^3)":                      0.0001785E+3,
   "Viscosity (Pa*s)":                      1.86e-05,
},
"Helium [He] (25 C)": {
   "Sound Velocity (m/s)":                  965.0,
   "-dV/dt (m/s*C)":                        -0.8,
},
"Hydrogen [H2] (0 C)": {
   "Melting Point (C)":                     -259.3,
   "Viscosity (Pa*s)":                      8.4e-06,
   "Density (kg/m^3)":                      8.99E+3,
   "Boiling Point (C)":                     -252.9,
},
"Hydrogen [H2] (25 C)": {
   "Sound Velocity (m/s)":                  1284.0,
   "-dV/dt (m/s*C)":                        -2.2,
},
"Iron (Cast)": {
   "Ultimate Stress (MPa)":                 [69.0, 480.0],
   "Poisson's Ratio":                       [0.2, 0.3],
   "Shear Modulus (GPa)":                   [32.0, 69.0],
   "Density (kg/m^3)":                      [7.0E+3, 7.4E+3],
   "Thermal Expansion Coefficient (1/C)":   [9.9E-6, 12.0E-6],
   "Elongation (%)":                        [0.0, 1.0],
   "Elastic Modulus (GPa)":                 [83.0, 170.0],
   "Yield Stress (MPa)":                    [120.0, 290.0],
},
"Iron (Wrought)": {
   "Ultimate Stress (MPa)":                 340.0,
   "Poisson's Ratio":                       0.3,
   "Shear Modulus (GPa)":                   75.0,
   "Density (kg/m^3)":                      [7.4E+3, 7.8E+3],
   "Thermal Expansion Coefficient (1/C)":   12.0E-6,
   "Elongation (%)":                        35.0,
   "Elastic Modulus (GPa)":                 190.0,
   "Yield Stress (MPa)":                    210.0,
},
"Iron [Fe]": {
   "Melting Point (C)":                     1538.0,
   "Thermal Conductivity (W/m*K)":          80.2,
   "Electric Resistivity (W*m)":            97.0E-9,
   "Density (kg/m^3)":                      7.87E+3,
   "Boiling Point (C)":                     2861.0,
},
"Iron [Fe] (Electrolytic/Armco)": {
   "Shear Wave Velocity (m/s)":              3240 / 3240,
   "Longitudinal Wave Velocity (m/s)":       5950 / 5960,
   "Bar Velocity (m/s)":                     5120 / 5200,
},
"Lead [Pb]": {
   "Melting Point (C)":                     327.5,
   "Thermal Conductivity (W/m*K)":          35.3,
   "Electric Resistivity (W*m)":            210.0E-9,
   "Density (kg/m^3)":                      11.3E+3,
   "Boiling Point (C)":                     1749.0,
},
"Lead [Pb] (Annealed/Rolled)": {
   "Shear Wave Velocity (m/s)":              700.0 / 690.0,
   "Longitudinal Wave Velocity (m/s)":       2160 / 1960,
   "Bar Velocity (m/s)":                     1190 / 1210,
},
"Magnesium Alloy": {
   "Boiling Point (C)":                     2061.0,
   "Ultimate Stress (MPa)":                 [140.0, 340.0],
   "Poisson's Ratio":                       0.35,
   "Shear Modulus (GPa)":                   17.0,
   "Density (kg/m^3)":                      1.77E+3,
   "Melting Point (C)":                     1246.0,
   "Thermal Expansion Coefficient (1/C)":   [26.1E-6, 28.8E-6],
   "Elongation (%)":                        [2.0, 20.0],
   "Elastic Modulus (GPa)":                 45.0,
   "Yield Stress (MPa)":                    [80.0, 280.0],
},
"Magnesium [Mg]": {
   "Boiling Point (C)":                     1090.0,
   "Electric Resistivity (W*m)":            45.0E-9,
   "Ultimate Stress (MPa)":                 [100.0, 170.0],
   "Poisson's Ratio":                       0.35,
   "Shear Modulus (GPa)":                   15.0,
   "Density (kg/m^3)":                      1.74E+3,
   "Melting Point (C)":                     650.0,
   "Thermal Expansion Coefficient (1/C)":   25.2E-6,
   "Elongation (%)":                        [5.0, 15.0],
   "Thermal Conductivity (W/m*K)":          156.0,
   "Elastic Modulus (GPa)":                 41.0,
   "Yield Stress (MPa)":                    [20.0, 70.0],
},
"Magnesium [Mg] (Drawn)": {
   "Shear Wave Velocity (m/s)":             3050.0,
   "Longitudinal Wave Velocity (m/s)":      5770.0,
   "Bar Velocity (m/s)":                    4940.0,
},
"Mercury [Hg] (20 C)": {
   "Boiling Point (C)":                     356.7,
   "Electric Resistivity (W*m)":            960.0E-9,
   "Viscosity (Pa*s)":                      0.00155,
   "Density (kg/m^3)":                      13.57904E+3,
   "Kinematic Viscosity (m2/s)":            1.14e-07,
   "Melting Point (C)":                     -38.83,
   "Bulk Modulus (GPa)":                    25.3,
   "Thermal Conductivity (W/m*K)":          8.25,
   "Thermal Expansion Coefficient (E-6/C)": 182.0,
},
"Mercury [Hg] (25 C)": {
   "Sound Velocity (m/s)":                  1450.0,
},
"Methane [CH4] (25 C)": {
   "Sound Velocity (m/s)":                  430.0,
},
"Monel (67% Ni, 30% Cu)": {
   "Longitudinal Wave Velocity (m/s)":      5350.0,
   "Ultimate Stress (MPa)":                 [450.0, 1200.0],
   "Poisson's Ratio":                       0.32,
   "Shear Modulus (GPa)":                   66.0,
   "Density (kg/m^3)":                      8.84E+3,
   "Melting Point (C)":                     1330.0,
   "Shear Wave Velocity (m/s)":             5720.0,
   "Thermal Expansion Coefficient (1/C)":   14.0E-6,
   "Elongation (%)":                        [2.0, 50.0],
   "Elastic Modulus (GPa)":                 170.0,
   "Bar Velocity (m/s)":                    4400.0,
   "Yield Stress (MPa)":                    [170.0, 1100.0],
},
"Neon [Ne] (0 C)": {
   "Melting Point (C)":                     -248.6,
   "Density (kg/m^3)":                      0.0008999E+3,
   "Boiling Point (C)":                     -246.1,
},
"Neon [Ne] (25 C)": {
   "Sound Velocity (m/s)":                  435.0,
   "-dV/dt (m/s*C)":                        -0.8,
},
"Nickel [Ni]": {
   "Boiling Point (C)":                     2913.0,
   "Longitudinal Wave Velocity (m/s)":      6040.0,
   "Electric Resistivity (W*m)":            70.0E-9,
   "Ultimate Stress (MPa)":                 [310.0, 760.0],
   "Poisson's Ratio":                       0.31,
   "Shear Modulus (GPa)":                   80.0,
   "Density (kg/m^3)":                      8.89E+3,
   "Melting Point (C)":                     1455.0,
   "Shear Wave Velocity (m/s)":             3000.0,
   "Thermal Expansion Coefficient (1/C)":   13.0E-6,
   "Elongation (%)":                        [2.0, 50.0],
   "Thermal Conductivity (W/m*K)":          90.7,
   "Elastic Modulus (GPa)":                 210.0,
   "Bar Velocity (m/s)":                    4900.0,
   "Yield Stress (MPa)":                    [140.0, 620.0],
},
"Nitrogen [N2] (0 C)": {
   "Melting Point (C)":                     -210.0,
   "Viscosity (Pa*s)":                      1.66e-05,
   "Density (kg/m^3)":                      0.00125E+3,
   "Boiling Point (C)":                     -195.8,
},
"Nitrogen [N2] (25 C)": {
   "Sound Velocity (m/s)":                  334.0,
   "-dV/dt (m/s*C)":                        -0.6,
},
"Nylon; Polyamide": {
   "Longitudinal Wave Velocity (m/s)":      2620.0,
   "Ultimate Stress (MPa)":                 [40.0, 70.0],
   "Poisson's Ratio":                       0.4,
   "Density (kg/m^3)":                      1.1E+3,
   "Shear Wave Velocity (m/s)":             1070.0,
   "Thermal Expansion Coefficient (1/C)":   [75.0E-6, 100.0E-6],
   "Elongation (%)":                        50.0,
   "Elastic Modulus (GPa)":                 [2.1, 2.8],
   "Bar Velocity (m/s)":                    1800.0,
},
"Oil; Lubricating (20 C)": {
   "Kinematic Viscosity (m2/s)":            0.0009,
   "Viscosity (Pa*s)":                      0.799,
},
"Oil; Mineral": {
   "Density (kg/m^3)":                      0.92E+3,
},
"Oil; Olive": {
   "Melting Point (C)":                     -6.0,
   "Density (kg/m^3)":                      0.92E+3,
},
"Oil; Petroleum": {
   "Density (kg/m^3)":                      0.82E+3,
},
"Oxygen [O2] (0 C)": {
   "Melting Point (C)":                     -218.8,
   "Viscosity (Pa*s)":                      1.92e-05,
   "Density (kg/m^3)":                      0.001429E+3,
   "Boiling Point (C)":                     -182.9,
},
"Oxygen [O2] (25 C)": {
   "Sound Velocity (m/s)":                  316.0,
   "-dV/dt (m/s*C)":                        -0.56,
},
"Platinum [Pt]": {
   "Boiling Point (C)":                     3825.0,
   "Longitudinal Wave Velocity (m/s)":      3260.0,
   "Electric Resistivity (W*m)":            110.0E-9,
   "Poisson's Ratio":                       0.38,
   "Density (kg/m^3)":                      21.4E+3,
   "Melting Point (C)":                     1768.0,
   "Shear Wave Velocity (m/s)":             1730.0,
   "Bar Velocity (m/s)":                    2800.0,
   "Thermal Conductivity (W/m*K)":          71.6,
   "Elastic Modulus (GPa)":                 145.0,
},
"Rubber": {
   "Longitudinal Wave Velocity (m/s)":      [1550.0, 1830.0],
   "Ultimate Stress (MPa)":                 [7.0, 20.0],
   "Poisson's Ratio":                       [0.45, 0.5],
   "Shear Modulus (GPa)":                   [0.0002, 0.001],
   "Density (kg/m^3)":                      [0.96E+3, 1.3E+3],
   "Thermal Expansion Coefficient (1/C)":   [130.0E-6, 200.0E-6],
   "Elongation (%)":                        [100.0, 800.0],
   "Elastic Modulus (GPa)":                 [0.0007, 0.004],
   "Yield Stress (MPa)":                    [1.0, 7.0],
},
"Silicon [Si]": {
   "Melting Point (C)":                     1382.0,
   "Thermal Conductivity (W/m*K)":          148.0,
   "Electric Resistivity (W*m)":            100.0E-9,
   "Density (kg/m^3)":                      2.33E+3,
},
"Silver [Ag]": {
   "Boiling Point (C)":                     2162.0,
   "Longitudinal Wave Velocity (m/s)":      3650.0,
   "Electric Resistivity (W*m)":            16.0E-9,
   "Density (kg/m^3)":                      10.49E+3,
   "Melting Point (C)":                     961.8,
   "Shear Wave Velocity (m/s)":             1610.0,
   "Bar Velocity (m/s)":                    2680.0,
   "Thermal Conductivity (W/m*K)":          429.0,
   "Elastic Modulus (GPa)":                 76.0,
},
"Solder; Tin-Lead": {
   "Electric Resistivity (W*m)":            [130.0E-9, 210.0E-9],
   "Ultimate Stress (MPa)":                 [12.0, 54.0],
   "Density (kg/m^3)":                      [8.17E+3, 11.34E+3],
   "Melting Point (C)":                     215.0,
   "Elongation (%)":                        [55.0, 30.0],
   "Thermal Conductivity (W/m*K)":          [30.0, 49.8],
   "Elastic Modulus (GPa)":                 [18.0, 35.0],
},
"Steel": {
   "Longitudinal Wave Velocity (m/s)":      5960.0,
   "Electric Resistivity (W*m)":            [120.0E-9, 1700.0E-9],
   "Ultimate Stress (MPa)":                 [340.0, 1900.0],
   "Poisson's Ratio":                       [0.27, 0.3],
   "Shear Modulus (GPa)":                   [75.0, 80.0],
   "Density (kg/m^3)":                      7.85E+3,
   "Melting Point (C)":                     1425.0,
   "Shear Wave Velocity (m/s)":             3235.0,
   "Thermal Expansion Coefficient (1/C)":   [10.0E-6, 18.0E-6],
   "Elongation (%)":                        [3.0, 40.0],
   "Elastic Modulus (GPa)":                 [190.0, 210.0],
   "Bar Velocity (m/s)":                    5200.0,
   "Yield Stress (MPa)":                    [280.0, 1600.0],
   "Specific Heat (J/kg*C)":                900,
},
"Stone; Granite": {
   "Density (kg/m^3)":                      2.6E+3,
},
"Stone; Granite (Compression)": {
   "Poisson's Ratio":                       [0.2, 0.3],
   "Elastic Modulus (GPa)":                 [40.0, 70.0],
   "Ultimate Stress (MPa)":                 [70.0, 280.0],
},
"Stone; Limestone": {
   "Density (kg/m^3)":                      [2.0E+3, 2.9E+3],
},
"Stone; Limestone (Compression)": {
   "Poisson's Ratio":                       [0.2, 0.3],
   "Elastic Modulus (GPa)":                 [20.0, 70.0],
   "Ultimate Stress (MPa)":                 [20.0, 200.0],
},
"Stone; Marble": {
   "Bar Velocity (m/s)":                    3810.0,
   "Density (kg/m^3)":                      [2.6E+3, 2.9E+3],
},
"Stone; Marble (Compression)": {
   "Poisson's Ratio":                       [0.2, 0.3],
   "Elastic Modulus (GPa)":                 [50.0, 100.0],
   "Ultimate Stress (MPa)":                 [50.0, 180.0],
},
"Stone; Quartz": {
   "Density (kg/m^3)":                      2.6E+3,
},
"Tin [Sn]": {
   "Boiling Point (C)":                     2602.0,
   "Longitudinal Wave Velocity (m/s)":      3320.0,
   "Electric Resistivity (W*m)":            110.0E-9,
   "Poisson's Ratio":                       0.36,
   "Density (kg/m^3)":                      7.3E+3,
   "Melting Point (C)":                     231.9,
   "Shear Wave Velocity (m/s)":             1670.0,
   "Bar Velocity (m/s)":                    2730.0,
   "Thermal Conductivity (W/m*K)":          66.6,
   "Elastic Modulus (GPa)":                 41.0,
},
"Titanium Alloy": {
   "Ultimate Stress (MPa)":                 [900.0, 970.0],
   "Poisson's Ratio":                       0.33,
   "Shear Modulus (GPa)":                   [39.0, 44.0],
   "Density (kg/m^3)":                      4.51E+3,
   "Thermal Expansion Coefficient (1/C)":   [8.0E-6, 10.0E-6],
   "Elongation (%)":                        10.0,
   "Elastic Modulus (GPa)":                 [110.0, 120.0],
},
"Titanium [Ti]": {
   "Boiling Point (C)":                     3287.0,
   "Longitudinal Wave Velocity (m/s)":      6070.0,
   "Electric Resistivity (W*m)":            430.0E-9,
   "Ultimate Stress (MPa)":                 500.0,
   "Poisson's Ratio":                       0.33,
   "Shear Modulus (GPa)":                   [40.0, 40.0],
   "Density (kg/m^3)":                      4.54E+3,
   "Melting Point (C)":                     1668.0,
   "Shear Wave Velocity (m/s)":             3125.0,
   "Elongation (%)":                        25.0,
   "Thermal Conductivity (W/m*K)":          21.9,
   "Elastic Modulus (GPa)":                 110.0,
   "Bar Velocity (m/s)":                    5080.0,
},
"Tungsten [W]": {
   "Boiling Point (C)":                     5555.0,
   "Electric Resistivity (W*m)":            56.0E-9,
   "Ultimate Stress (MPa)":                 [1400.0, 4000.0],
   "Density (kg/m^3)":                      19.3E+3,
   "Melting Point (C)":                     3422.0,
   "Thermal Expansion Coefficient (1/C)":   4.3E-6,
   "Elongation (%)":                        [0.0, 4.0],
   "Thermal Conductivity (W/m*K)":          174.0,
},
"Tungsten [W] (Annealed/Drawn)": {
   "Shear Wave Velocity (m/s)":              2890 / 2640,
   "Longitudinal Wave Velocity (m/s)":       5220 / 5410,
   "Bar Velocity (m/s)":                     4620 / 4320,
},
"Water; Distilled [H2O] (20 C)": {
   "Boiling Point (C)":                     100.0,
   "Viscosity (Pa*s)":                      0.001,
   "Density (kg/m^3)":                      0.998E+3,
   "Kinematic Viscosity (m2/s)":            1e-06,
   "Melting Point (C)":                     0.0,
   "Sound Velocity (m/s)":                  1481.0,
   "Bulk Modulus (GPa)":                    2.18,
   "Thermal Expansion Coefficient (E-6/C)": 207.0,
},
"Water; Distilled [H2O] (25 C)": {
   "Boiling Point (C)":                     100.0,
   "Density (kg/m^3)":                      0.997E+3,
   "-dV/dt (m/s*C)":                        -2.4,
   "Melting Point (C)":                     0.0,
   "Sound Velocity (m/s)":                  1497.0,
   "Thermal Conductivity (W/m*K)":          0.607,
},
"Water; Distilled [H2O] (4 C)": {
   "Kinematic Viscosity (m2/s)":            1.57e-06,
   "Melting Point (C)":                     0.0,
   "Viscosity (Pa*s)":                      0.00157,
   "Density (kg/m^3)":                      1.0E+3,
   "Boiling Point (C)":                     100.0,
},
"Water; Sea (13 C)": {
   "Bulk Modulus (GPa)":                    2.28,
   "Sound Velocity (m/s)":                  1500.0,
   "Density (kg/m^3)":                      1.024E+3,
   "-dV/dt (m/s*C)":                        -2.4,
},
"Water; Steam [H2O] (100 C)": {
   "Sound Velocity (m/s)":                  404.8,
   "Density (kg/m^3)":                      0.6E+3,
},
"Wood; Ash": {
   "Density (kg/m^3)":                      [0.56E+3, 0.64E+3],
},
"Wood; Ash (Along fiber/ring)": {
   "Bar Velocity (m/s)":                     4670 / 1260,
},
"Wood; Ash (Bending)": {
   "Elastic Modulus (GPa)":                 [10.0, 11.0],
   "Yield Stress (MPa)":                    [40.0, 70.0],
   "Ultimate Stress (MPa)":                 [50.0, 100.0],
},
"Wood; Douglas Fir": {
   "Density (kg/m^3)":                      [0.48E+3, 0.56E+3],
},
"Wood; Douglas Fir (Bending)": {
   "Elastic Modulus (GPa)":                 [11.0, 13.0],
   "Yield Stress (MPa)":                    [30.0, 50.0],
   "Ultimate Stress (MPa)":                 [50.0, 80.0],
},
"Wood; Oak": {
   "Density (kg/m^3)":                      [0.64E+3, 0.72E+3],
},
"Wood; Oak (Along fiber)": {
   "Bar Velocity (m/s)":                    3850.0,
},
"Wood; Oak (Bending)": {
   "Elastic Modulus (GPa)":                 [11.0, 12.0],
   "Yield Stress (MPa)":                    [40.0, 60.0],
   "Ultimate Stress (MPa)":                 [50.0, 100.0],
},
"Wood; Southern Pine": {
   "Density (kg/m^3)":                      [0.55E+3, 0.64E+3],
},
"Wood; Southern Pine (Bending)": {
   "Elastic Modulus (GPa)":                 [11.0, 14.0],
   "Yield Stress (MPa)":                    [40.0, 60.0],
   "Ultimate Stress (MPa)":                 [50.0, 100.0],
},
"Zinc [Zn]": {
   "Boiling Point (C)":                     907.0,
   "Electric Resistivity (W*m)":            59.0E-9,
   "Poisson's Ratio":                       0.25,
   "Density (kg/m^3)":                      7.14E+3,
   "Melting Point (C)":                     419.5,
   "Thermal Expansion Coefficient (1/C)":   30.2E-6,
   "Thermal Conductivity (W/m*K)":          116.0,
},
"Zinc [Zn] (Rolled)": {
   "Shear Wave Velocity (m/s)":             2440.0,
   "Longitudinal Wave Velocity (m/s)":      4210.0,
   "Bar Velocity (m/s)":                    3850.0,
},
}



#!/usr/bin/env python

"""
Demonstration of the quiver command in combination with other plotting
commands.
"""

from scitools.std import *

setp(interactive=False)

xv, yv = ndgrid(linspace(-5,5,81), linspace(-5,5,81))
values = sin(sqrt(xv**2 + yv**2))

pcolor(xv, yv, values, shading='interp')

# create a coarser grid for the gradient field:
xv, yv = ndgrid(linspace(-5,5,21), linspace(-5,5,21), sparse=True)
values = sin(sqrt(xv**2 + yv**2))

# compute the gradient field:
uu, vv = gradient(values)

hold('on')
quiver(xv, yv, uu, vv, 'filled', 'k', axis=[-6,6,-6,6])
show()

#hardcopy('quiver2a.eps')
#hardcopy('quiver2a.png')


figure()
contour(xv, yv, values, 15, hold=True)
quiver(xv, yv, uu, vv, axis=[-6,6,-6,6])
show()

#hardcopy('quiver2b.eps')
#hardcopy('quiver2b.png')

raw_input("Press Return key to quit: ")



#!/usr/bin/env python

import os, glob, re

from scitools.misc import findprograms
from misc import _check_type

class MovieEncoder(object):
    """
    Class for turning a series of filenames with frames in a movie into
    a movie file.
    """
    _local_prop = {
        'input_files': None,         # image files
        'output_file': None,         # resulting movie
        'overwrite_output': True,    # overwrite output file if True
        'encoder': None,             # encoding tool
        'vbitrate': None,            # video bit rate
        'vbuffer': None,             # video buffer
        'fps': 25,                   # default frame rate
        'vcodec': 'mpeg4',           # default video codec
        'size': None,                # final movie size
        'quiet': False,              # run in quite mode
        'aspect': None,              # aspect ratio
        'preferred_package': 'ImageMagick',  # prefer ImageMagick or Netpbm
        'qscale': None,              # quantization scale
        'qmin': 2,                   # minimum quantization scale
        'qmax': 31,                  # maximum quantization scale
        'iqscale': None,             # quantization scale for I frames
        'pqscale': None,             # quantization scale for P frames
        'bqscale': None,             # quantization scale for B frames
        'pattern': 'I',              # pattern for I, P, and B frames
        'gop_size': None,            # size of GOP (group of pictures)
        'force_conversion': False,   # force conversion (to png) if True
        'cleanup': True,             # clean up temporary files
        }
    _legal_encoders = 'convert mencoder ffmpeg mpeg_encode ppmtompeg '\
                      'mpeg2enc html'.split()
    _legal_file_types = 'png gif jpg ps eps bmp tif tga pnm'.split()

    def __init__(self, input_files, **kwargs):
        self._prop = {}
        self._prop.update(self._local_prop)
        self._prop['input_files'] = input_files
        for key in kwargs:
            if key in self._prop:
                self._prop[key] = kwargs[key]

        print '\n\n' # provide some space before print statements

        # Determine which encoder to be used
        encoder = self._prop['encoder']
        if encoder is None:
            # No encoder given, find the first installed among the legal ones
            for enc in self._legal_encoders:
                if findprograms(enc):
                    encoder = enc
                    break
            if encoder is None:
                # No encoder is installed, fall back on html
                encoder = 'html'
            self._prop['encoder'] = encoder
        else:
            if not encoder in self._legal_encoders:
                raise ValueError("encoder must be %s, not '%s'" %
                                 (self._legal_encoders, encoder))
            if not encoder.startswith('html') and not findprograms(encoder):
                raise Exception("The selected encoder (%s) is not installed" \
                                % encoder)

        # Determine the file type of the input files
        if isinstance(input_files, (tuple,list)):
            file_ = input_files[0]
        elif isinstance(input_files, str):
            file_ = input_files
        else:
            raise ValueError("The input files must be given as either a "\
                             "list/tuple of strings or a string, not '%s'" % \
                             type(input_files))

        # Check that the input files do exist
        if isinstance(input_files, str):
            # Are input_files on ffmpeg/mpeg2enc format or Unix wildcard format?
            ffmpeg_format = r'(.+)%\d+d(\..+)'
            m = re.search(ffmpeg_format, input_files, re.DOTALL)
            if m:
                wildcard_format = m.group(1) + '*' + m.group(2)
            else:
                wildcard_format = input_files
            all_input_files = glob.glob(wildcard_format)
            if not all_input_files:
                print 'No files of the form %s exist.' % input_files
            else:
                print 'Found %d files of the format %s.' % \
                (len(all_input_files), input_files)
        else:  # list of specific filenames
            all_input_files = input_files
        error_encountered = False
        for f in all_input_files:
            if not os.path.isfile(f):
                print 'Input file %s does not exist.' % f
                error_encountered = True
        if error_encountered:
            raise IOError('Some input files were not found.')

        fname, ext = os.path.splitext(file_)
        if not ext:
            raise ValueError("Unable to determine file type from file name.")
        file_type = ext[1:] # remove the . (dot)
        if not file_type in self._legal_file_types:
            raise TypeError("File type must be %s, not '%s'" % \
                            (self._legal_file_types, file_type))
        self._prop['file_type'] = file_type

    def encode(self):
        """Encode a series of images to a movie."""
        encoder = self._prop['encoder']

        if encoder == 'html':
            # Don't make movie file, just an html file that can play png files
            files = self._prop['input_files']
            if isinstance(files, str):
                files = glob.glob(files)
                files.sort()
            print '\nMaking HTML code for displaying', ', '.join(files)
            fps = self._prop['fps']
            interval_ms = 1000.0/fps
            outf = self._prop['output_file']
            if outf is None:
                outf = 'movie.html'
            else:
                # Ensure .html extension
                outf = os.path.splitext(outf)[0] + '.html'
            casename = os.path.splitext(outf)[0]
            header, jscode, form, footer = \
                html_movie(files, interval_ms, casename=casename)

            f = open(outf, 'w')
            f.write(header + jscode + form + footer)
            f.close()
            print "\n\nmovie in output file", outf
            return

        # Get command string (all other encoders are run as stand-alone apps)
        exec('cmd=self._%s()' % encoder)

        # Run command
        if not self._prop['quiet']:
            print "\nscitools.easyviz.movie function runs the command: \n\n%s\n" % cmd
        failure = os.system(cmd)
        if failure:
            print '\n\nscitools.easyviz.movie could not make movie'
            raise SystemError('Check error messages from the encoder in the terminal window')
        elif not self._prop['quiet']:
            print "\n\nmovie in output file", self._prop['output_file']

        # Clean up temporary files
        if self._prop['cleanup'] and hasattr(self, '_tmp_files'):
            for tmp_file in self._tmp_files:
                os.remove(tmp_file)

    def _convert(self):
        """Return a string with commands for making a movie with the convert
        tool (from ImageMagick)."""
        encoder = self._prop['encoder']
        cmd = encoder

        # set number of frames per second:
        #cmd += ' -delay 1x%s' % self._prop['fps']
        cmd += ' -delay %d' % (1.0/self._prop['fps']*100)

        # set size:
        size = self._get_size()
        if size is not None:
            cmd += ' -scale %sx%s' % (size[0], size[1])

        # get image files:
        files = self._prop['input_files']
        if isinstance(files, str):
            pattern = r'(.*)%(\d+)d(.*\..*)'
            match = re.search(pattern, files)
            if match:
                pre = match.group(1)
                num = int(match.group(2))
                ext = match.group(3)
                files = pre + '[0-9]'*num + ext
            files = glob.glob(files)
            files.sort()
        if not files:
            raise ValueError(
                "'%s' is not a valid file specification or the files " \
                "does not exist." % files)
        if isinstance(self._prop['input_files'], str):
            cmd += ' ' + self._prop['input_files']
        else:
            cmd += ' %s' % (' '.join(files))

        # set output file:
        if self._prop['output_file'] is None:
            self._prop['output_file'] = 'movie.gif'
        output_file = self._prop['output_file']
        if output_file[-4:] != '.gif':
            output_file += '.gif'
        if os.path.isfile(output_file) and not self._prop['overwrite_output']:
            raise Exception("Output file '%s' already exist. Use" \
                            " 'overwrite_output=True' to overwrite the file." \
                            % output_file)
        cmd += ' %s' % output_file

        return cmd

    def _mencoder(self):
        """Return a string with commands for making a movie with the MEncoder
        tool.
        """
        encoder = self._prop['encoder']
        cmd = encoder
        file_type = self._prop['file_type']
        files = self._prop['input_files']
        if isinstance(files, str):
            pattern = r'(.*)%(\d+)d(.*\..*)'
            match = re.search(pattern, files)
            if match:
                pre = match.group(1)
                num = int(match.group(2))
                ext = match.group(3)
                files = pre + '[0-9]'*num + ext
            files = glob.glob(files)
            files.sort()
            if not files:
                raise ValueError(
                    "'%s' is not a valid file specification or the files "\
                    "does not exist." % self._prop['input_files'])
        if isinstance(files, (list,tuple)):
            if not file_type in ['jpg', 'png'] or \
                   self._prop['force_conversion']:
                # since mencoder only supports jpg and png files, we have to
                # create copies of the input files and convert them to jpg.
                files = self._any2any(files, ofile_ext='.png')
                file_type = 'png'
                self._tmp_files = files[:] # store files for later removal
            # give the files as a comma separated string to mencoder:
            files = ','.join(files)
        cmd += ' "mf://%s" -mf' % files
        cmd += ' fps=%g' % float(self._prop['fps'])
        cmd += ':type=%s' % file_type
        vbitrate = self._prop['vbitrate']
        if vbitrate is None:
            vbitrate = 800
        vcodec = self._prop['vcodec']
        qscale = self._prop['qscale']
        qmin = self._prop['qmin']
        qmax = self._prop['qmax']
        vbuffer = self._prop['vbuffer']
        if vcodec == 'xvid':
            cmd += ' -ovc xvid -xvidencopts'
            if qscale is not None:
                cmd += ' fixed_quant=%s' % qscale
            else:
                cmd += ' bitrate=%s:' % vbitrate
                cmd += 'min_iquant=%s:max_iquant=%s:' % (qmin, qmax)
                cmd += 'min_pquant=%s:max_pquant=%s:' % (qmin, qmax)
                cmd += 'min_bquant=%s:max_bquant=%s' % (qmin, qmax)
        else:
            # mbd: macroblock decision
            cmd += ' -ovc lavc -lavcopts vcodec=%s:mbd=1' % vcodec
            if vbitrate is not None:
                cmd += ':vbitrate=%s' % vbitrate
                #cmd += ':vrc_minrate=%s:vrc_maxrate=%s' % ((vbitrate,)*2)
            if qscale is not None:
                cmd += ':vqscale=%s' % qscale
            else:
                cmd += ':vqmin=%s:vqmax=%s' % (qmin, qmax)
            if vbuffer is not None:
                cmd += ':vrc_buf_size=%s' % vbuffer
        aspect = self._prop['aspect']
        if aspect is not None:
            cmd += ':aspect=%s' % aspect
        #cmd += ' -oac copy' # audio
        size = self._get_size()
        if size is not None:
            cmd += ' -vf scale=%s:%s' % (size[0], size[1])
        # if no output file is given, use 'movie.avi' as default:
        if self._prop['output_file'] is None:
            self._prop['output_file'] = 'movie.avi'
        output_file = self._prop['output_file']
        if os.path.isfile(output_file) and not self._prop['overwrite_output']:
            raise Exception("Output file '%s' already exist. Use" \
                            " 'overwrite_output=True' to overwrite the file." \
                            % output_file)
        cmd += ' -o %s' % output_file
        if self._prop['quiet']:
            cmd += ' > /dev/null 2>&1'
        return cmd

    def _ffmpeg(self):
        """Return a string with commands for making a movie with the FFmpeg
        tool.
        """
        encoder = self._prop['encoder']
        cmd = encoder
        files = self._prop['input_files']
        file_type = self._prop['file_type']
        if isinstance(files, str):
            pattern = r'(.*)%(\d+)d(.*\..*)'
            match = re.search(pattern, files)
            if match:
                if file_type not in ['jpg', 'png'] or \
                       self._prop['force_conversion']:
                    pre = match.group(1)
                    num = int(match.group(2))
                    ext = match.group(3)
                    files = pre + '[0-9]'*num + ext
                    files = glob.glob(files)
                    files.sort()
            else:
                files = glob.glob(files)
                files.sort()
        if isinstance(files, (list,tuple)):
            basename = 'easyviz_tmp_'
            files = self._any2any(files, basename=basename, ofile_ext='.png')
            file_type = 'png'
            self._tmp_files = files[:]
            # create a new string with the right pattern:
            files = basename + '%04d.png'
        cmd += ' -i "%s"' % files
        vbitrate = self._prop['vbitrate']
        if vbitrate is None:
            vbitrate = "800k"
        cmd += ' -b %s' % vbitrate
        cmd += ' -r %s' % self._prop['fps']
        size = self._get_size()
        if size is not None:
            cmd += ' -s %sx%s' % size
        if self._prop['vcodec'] is not None:
            cmd += ' -vcodec %s' % self._prop['vcodec']
        if self._prop['overwrite_output']:
            cmd += ' -y'
        if self._prop['aspect'] is not None:
            cmd += ' -aspect %s' % self._prop['aspect']
        if self._prop['qscale'] is not None:
            cmd += ' -qscale %s' % self._prop['qscale']
        else:
            cmd += ' -qmin %s -qmax %s' % \
                   (self._prop['qmin'], self._prop['qmax'])
        if self._prop['vbuffer'] is not None:
            cmd += ' -bufsize %s' % self._prop['vbuffer']
        if self._prop['gop_size'] is not None:
            cmd += ' -g %d' % int(self._prop['gop_size'])
        #cmd += ' -target dvd'
        # if no output file is given, use 'movie.avi' as default:
        if self._prop['output_file'] is None:
            self._prop['output_file'] = 'movie.avi'
        cmd += ' ' + self._prop['output_file']
        if self._prop['quiet']:
            cmd += ' > /dev/null 2>&1'
        return cmd

    def _mpeg_encode(self):
        """Return a string with commands for making a movie with the
        mpeg_encode tool.
        """
        encoder = self._prop['encoder']
        basename = 'easyviz_tmp_'  # basename for temporary files

        # set frame rate:
        # mpeg_encode only supports a given set of frame rates:
        legal_frame_rates = (23.976, 24, 25, 29.97, 30, 50, 59.94, 60)
        if self._prop['fps'] in legal_frame_rates:
            fps = self._prop['fps']
        else:
            raise ValueError("%s only supports the these frame rates: %s" % \
                             (encoder, legal_frame_rates))

        # set aspect ratio:
        legal_aspects = (1.0, 0.6735, 0.7031, 0.7615, 0.8055,
                         0.8437, 0.8935, 0.9157, 0.9815, 1.0255,
                         1.0695, 1.0950, 1.1575, 1.2015)
        aspect = self._prop['aspect']
        if aspect is not None:
            if aspect not in legal_aspects:
                raise(ValueError, \
                      "%s only supports the following aspect ratios: %s" % \
                      (encoder, legal_aspects))
        else:
            aspect = 1.0
        print aspect

        # get image files:
        files = self._prop['input_files']
        if isinstance(files, str):
            pattern = r'(.*)%(\d+)d(.*\..*)'
            match = re.search(pattern, files)
            if match:
                pre = match.group(1)
                num = int(match.group(2))
                ext = match.group(3)
                files = pre + '[0-9]'*num + ext
            files = glob.glob(files)
            files.sort()
        if not files:
            raise ValueError(
                "'%s' is not a valid file specification or the files " \
                "does not exist." % files)
        size = self._get_size()
        if size is not None or self._prop['file_type'] != 'pnm' or \
               self._prop['force_conversion']:
            files = self._any2any(files, basename=basename, size=size)
            self._tmp_files = files[:]
        files = '\n'.join(files)

        # set input dir:
        input_dir = os.path.split(files[0])[0]
        if input_dir == '':
            input_dir = '.'

        # set output file:
        if self._prop['output_file'] is None:
            self._prop['output_file'] = 'movie.mpeg'
        mpeg_file = self._prop['output_file']
        if os.path.isfile(mpeg_file) and not self._prop['overwrite_output']:
            raise Exception("Output file '%s' already exist. Use" \
                            " 'overwrite_output=True' to overwrite the file." \
                            % mpeg_file)

        # set pattern (sequence of I, P, and B frames):
        pattern = self._prop['pattern']
        if isinstance(pattern, str):
            pattern = pattern.upper()  # pattern must be uppercase
        else:
            # give warning?
            pattern = 'I'

        # set quantization scale for I, P, and B frames:
        qscale = self._prop['qscale']
        iqscale = self._prop['iqscale']
        pqscale = self._prop['pqscale']
        bqscale = self._prop['bqscale']
        if qscale is not None:
            if iqscale is None: iqscale = qscale
            if pqscale is None: pqscale = qscale
            if bqscale is None: bqscale = qscale
        else:
            if iqscale is None: iqscale = 8
            if pqscale is None: pqscale = 10
            if bqscale is None: bqscale = 25

        # set gop size:
        gop_size = self._prop['gop_size']
        if gop_size is not None:
            gop_size = int(gop_size)
        else:
            gop_size = 15

        # create an mpeg_encode parameter file:
        mpeg_encode_file = "%s.mpeg_encode-input" % basename
        f = open(mpeg_encode_file, "w")
        f.write("""
PATTERN	         %(pattern)s
OUTPUT           %(mpeg_file)s
BASE_FILE_FORMAT PNM
INPUT_CONVERT    *
GOP_SIZE         %(gop_size)d
#GOP_SIZE         16
SLICES_PER_FRAME 1
INPUT_DIR        %(input_dir)s
INPUT
%(files)s
END_INPUT
PIXEL            FULL
#PIXEL            HALF
RANGE            10
PSEARCH_ALG      LOGARITHMIC
BSEARCH_ALG      CROSS2
IQSCALE	         %(iqscale)d
PQSCALE	         %(pqscale)d
BQSCALE	         %(bqscale)d
REFERENCE_FRAME  ORIGINAL
FRAME_RATE       %(fps)d
ASPECT_RATIO     %(aspect)s
FORCE_ENCODE_LAST_FRAME
""" % vars())

        # set video bit rate and buffer size:
        vbitrate = self._prop['vbitrate']
        if isinstance(vbitrate, (float,int)):
            f.write("BIT_RATE         %d" % (int(vbitrate)*1000))
        vbuffer = self._prop['vbuffer']
        if isinstance(vbuffer, (float,int)):
            f.write("BUFFER_SIZE      %d" % (int(vbuffer)*1000))
        f.close()

        if not hasattr(self, '_tmp_files'):
            self._tmp_files = []
        self._tmp_files.append(mpeg_encode_file)

        # create the command string:
        cmd = encoder
        if self._prop['quiet']:
            cmd += ' -realquiet'
        cmd += ' ' + mpeg_encode_file
        return cmd

    # mpeg_encode and ppmtompeg is the same encoding tool:
    _ppmtompeg = _mpeg_encode

    def _mpeg2enc(self):
        """Return a string with commands for making a movie with the
        mpeg2enc tool (from MJPEGTools).
        """
        encoder = self._prop['encoder']
        png2yuv = 'png2yuv'
        jpeg2yuv = 'jpeg2yuv'
        yuvscaler = 'yuvscaler'

        file_type = self._prop['file_type']
        files = self._prop['input_files']
        if isinstance(files, str):
            pattern = r'(.*)%(\d+)d(.*\..*)'
            match = re.search(pattern, files)
            if match:
                if file_type not in ['jpg', 'png'] or \
                       self._prop['force_conversion']:
                    pre = match.group(1)
                    num = int(match.group(2))
                    ext = match.group(3)
                    files = pre + '[0-9]'*num + ext
                    files = glob.glob(files)
                    files.sort()
            else:
                files = glob.glob(files)
                files.sort()
        if isinstance(files, (list,tuple)):
            basename = 'easyviz_tmp_'
            files = self._any2any(files, basename=basename, ofile_ext='.png')
            file_type = 'png'
            self._tmp_files = files[:]
            # create a new string with the right pattern:
            files = basename + '%04d.png'

        cmd = ''
        if file_type == 'jpg' and findprograms(jpeg2yuv):
            cmd += jpeg2yuv
        elif findprograms(png2yuv):
            cmd += png2yuv
        else:
            raise Exception("png2yuv or jpeg2yuv is not installed")
        cmd += ' -f 25' # frame rate
        cmd += ' -I p'  # interlacing mode: p = none / progressive
        cmd += ' -j "%s"' % files # set image files
        # find start image:
        for i in xrange(9999):
            if os.path.isfile(files % i):
                cmd += ' -b %d' % i
                break
        if self._prop['quiet']:
            cmd += ' -v 0' # verbosity level 0

        # set size of movie (by using the yuvscaler tool):
        size = self._get_size()
        if size is not None and findprograms(yuvscaler):
            width, height = size
            cmd += ' | %(yuvscaler)s -O SIZE_%(width)sx%(height)s' % vars()

        # if no output file is given, use 'movie.avi' as default:
        if self._prop['output_file'] is None:
            self._prop['output_file'] = 'movie.mpeg'
        output_file = self._prop['output_file']
        if os.path.isfile(output_file) and not self._prop['overwrite_output']:
            raise Exception("Output file '%s' already exist. Use" \
                            " 'overwrite_output=True' to overwrite the file." \
                            % output_file)

        cmd += ' | '
        cmd += encoder
        if self._prop['vcodec'] == 'mpeg2video':
            cmd += ' -f 3' # generic mpeg-2 video
        else:
            cmd += ' -f 0' # generic mpeg-1 video
        if self._prop['vbitrate'] is not None:
            cmd += ' -b %d' % int(self._prop['vbitrate'])
        if self._prop['vbuffer'] is not None:
            cmd += ' -V %d' % int(self._prop['vbuffer'])
        if self._prop['qscale'] is not None:
            cmd += ' -q %s' % self._prop['qscale']

        # set movie frame rate:
        legal_fps = {'23.976': 1, '24': 2, '25': 3, '29.97': 4,
                     '30': 5, '50': 6, '59.94': 7, '60': 8}
        fps = str(self._prop['fps'])
        if not fps in legal_fps:
            raise ValueError("fps must be %s, not %s" % \
                             (fps_convert.keys(), fps))
        cmd += ' -F %s' % legal_fps[fps]
        #cmd += ' --cbr' # constant bit rate
        gop_size = self._prop['gop_size']
        if gop_size is not None:
            # set min (-g) and max (-G) gop size to the same value:
            cmd += ' -g %s -G %s' % (gop_size, gop_size)

        # set aspect ratio:
        legal_aspects = {'1.0': 1, '1.3': 2, '1.7': 3, '2.21': 4}
        aspect = self._get_aspect_ratio()
        if aspect is not None:
            if aspect not in legal_aspects.values():
                aspect = str(aspect)
                for key in legal_aspects.keys():
                    if aspect.startswith(key):
                        aspect = legal_aspects[key]
                        break
                if aspect not in legal_aspects.values():
                    raise ValueError(
                        "aspect must be either 1:1, 4:3, 16:9, or 2.21:1," \
                        " not '%s'" % aspect)
            cmd += ' -a %s' % aspect

        # set output file:
        cmd += ' -o %s' % self._prop['output_file']
        if self._prop['quiet']:
            cmd += ' -v 0' # verbosity level 0 (warnings and errors only)
        return cmd

    def _any2any(self, files, basename='easyviz_tmp_',
                 size=None, ofile_ext='.pnm'):
        """Convert a list of files to the file format specified in the
        ofile_ext keyword argument. Using either Netpbm tools or convert
        (from the ImageMagick package).
        """
        netpbm_converters = {'.png': ('pngtopnm', 'pnmtopng'),
                             '.gif': ('giftopnm',  'ppmtogif'),
                             '.jpg': ('jpegtopnm', 'pnmtojpeg'),
                             '.ps':  ('pstopnm', 'pnmtops'),
                             '.eps': ('pstopnm', 'pnmtops'),
                             '.bmp': ('bmptopnm', 'ppmtobmp'),
                             '.tif': ('tifftopnm', 'pnmtotiff'),
                             '.tga': ('tgatopnm', 'ppmtotga'),
                             '.pnm': ('cat', ''),
                             }
        _check_type(files, 'files', (list,tuple))
        ifile_ext = os.path.splitext(files[0])[1]
        anytopnm = netpbm_converters[ifile_ext][0]
        pnmtoany = netpbm_converters[ofile_ext][1]
        pnmscale = 'pnmscale'
        #pnmcrop = 'pnmcrop'
        convert = 'convert'

        app = anytopnm
        if findprograms((convert, anytopnm, pnmtoany)):
            if self._prop['preferred_package'].lower() == 'imagemagick':
                app = convert
        elif findprograms(convert):
            app = convert
        elif not findprograms((anytopnm, pnmtoany)):
            raise Exception("Neither %s nor %s was found" % (convert,anytopnm))

        quiet = self._prop['quiet']
        new_files = []
        i = 1 # counter
        for file_ in files:
            new_file = "%s%04d%s" % (basename, i, ofile_ext)
            if app == anytopnm:
                options = ''
                if quiet and app != 'cat':
                    options += '-quiet'
                if app == 'pstopnm':
                    options += ' -stdout'
                    #options += ' -portrait'
                cmd = "%(app)s %(options)s %(file_)s " % vars()
                if size is not None and findprograms(pnmscale):
                    w, h = size
                    cmd += "| %(pnmscale)s -width %(w)s -height %(h)s" % vars()
                if pnmtoany != '':
                    options = ''
                    if quiet:
                        options += '-quiet'
                    if pnmtoany == 'pnmtojpeg':
                        options += ' -quality 100' # don't lose quality
                    cmd += " | %(pnmtoany)s %(options)s" % vars()
                cmd += " > %s" % new_file
            else:
                options = ''
                if size is not None:
                    options += '-resize %sx%s' % size
                cmd = "%(app)s %(options)s %(file_)s %(new_file)s" % vars()
            if not quiet:
                print cmd
            failure = os.system(cmd)
            if failure:
                print "... %s failed, jumping to next file..." % app
                continue
            new_files.append(new_file)
            if not quiet:
                apps = app
                if app != convert and pnmtoany != '':
                    apps += ' and %s' % pnmtoany
                print "%s transformed via %s to %s (%d Kb)" % \
                      (file_,apps,new_file,int(os.path.getsize(new_file)/1000))
            i += 1

        return new_files

    def _get_aspect_ratio(self):
        """Parse and return the aspect ratio."""
        # accept aspect ratio on the form 4:3, 4/3, or 1.3333
        aspect = self._prop['aspect']
        if isinstance(aspect, str):
            if aspect.find(':') > 0:
                aspect = aspect.split(':')
            elif aspect.find('/'):
                aspect = aspect.split('/')
            else:
                try: aspect = float(aspect)
                except: aspect = None
            try: aspect = float(aspect[0]) / float(aspect[1])
            except: aspect = None
        return aspect

    def _get_size(self):
        """Parse and return the size."""
        legal_sizes = {'sqcif': (128, 96),
                       'qcif': (176, 144),
                       'cif': (352, 288),
                       '4cif': (704, 576)}
        size = self._prop['size']
        if isinstance(size, str):
            if size in legal_sizes:
                size = legal_sizes[size]
            else:
                size = size.split('x') # wxh
        if not (isinstance(size, (tuple,list)) and len(size) == 2):
            size = None
        return size

def html_movie(plotfiles, interval_ms=300, width=800, height=600,
               casename='movie'):
    """
    Takes a list plotfiles which should be for example of the form:
        ['frame00.png', 'frame01.png', ... ]
    where each string should be the name of an image file and they should be
    in the proper order for viewing as an animation.

    The result is html text strings that incorporate javascript to
    loop through the plots one after another.  The html text also features
    buttons for controlling the movie.
    The parameter iterval_ms is the time interval between loading
    successive images and is in milliseconds.

    The width and height parameters do not seem to have any effect
    for reasons not understood.

    The following strings are returned: header, javascript code, form
    with movie and buttons, and footer. Concatenating these strings
    and dumping to an html file yields a kind of movie file to be
    viewed in a browser. The images variable in the javascript code
    is unique for each movie, because it is annotated by the casename
    string, so several such javascript sections can be used in the
    same html file.

    This function is based on code written by R.J. LeVeque, based on
    a template from Alan McIntyre.
    """
    import os
    if not isinstance(plotfiles, (tuple,list)):
        raise TypeError('html_movie: plotfiles=%s of wrong type %s' %
                        (str(plotfiles), type(plotfiles)))
    # Check that the plot files really exist
    missing_files = [fname for fname in plotfiles if not os.path.isfile(fname)]
    if missing_files:
        raise ValueError('Missing plot files: %s' % str(missing_files)[1:-1])

    ext = os.path.splitext(plotfiles[0])[-1]
    if ext == '.png' or ext == '.jpg' or ext == '.jpeg' or ext == 'gif':
        pass
    else:
        raise ValueError('Plotfiles (%s, ...) must be PNG files with '\
                         'extension .png' % plotfiles[0])
    header = """\
<html>
<head>
</head>
<body>
"""
    no_images = len(plotfiles)
    jscode = """
<script language="Javascript">
<!---
var num_images_%(casename)s = %(no_images)d;
var img_width = %(width)d;
var img_height = %(height)d;
var interval = %(interval_ms)d;
var images_%(casename)s = new Array();

function preload_images_%(casename)s()
{
   t = document.getElementById("progress");
""" % vars()

    i = 0
    for fname in plotfiles:
        jscode += """
   t.innerHTML = "Preloading image ";
   images_%(casename)s[%(i)s] = new Image(img_width, img_height);
   images_%(casename)s[%(i)s].src = "%(fname)s";
        """ % vars()
        i = i+1
    jscode += """
   t.innerHTML = "";
}

function tick_%(casename)s()
{
   if (frame_%(casename)s > num_images_%(casename)s - 1)
       frame_%(casename)s = 0;

   document.movie.src = images_%(casename)s[frame_%(casename)s].src;
   frame_%(casename)s += 1;
   tt = setTimeout("tick_%(casename)s()", interval);
}

function startup_%(casename)s()
{
   preload_images_%(casename)s();
   frame_%(casename)s = 0;
   setTimeout("tick_%(casename)s()", interval);
}

function stopit()
{ clearTimeout(tt); }

function restart_%(casename)s()
{ tt = setTimeout("tick_%(casename)s()", interval); }

function slower()
{ interval = interval/0.7; }

function faster()
{ interval = interval*0.7; }

// --->
</script>
""" % vars()
    plotfile0 = plotfiles[0]
    form = """
<form>
&nbsp;
<input type="button" value="Start movie" onClick="startup_%(casename)s()">
<input type="button" value="Pause movie" onClick="stopit()">
<input type="button" value="Restart movie" onClick="restart_%(casename)s()">
&nbsp;
<input type="button" value="Slower" onClick="slower()">
<input type="button" value="Faster" onClick="faster()">
</form>

<p><div ID="progress"></div></p>
<img src="%(plotfile0)s" name="movie" border=2/>
""" % vars()
    footer = '\n</body>\n</html>\n'
    return header, jscode, form, footer


def movie(input_files, **kwargs):
    """
    Make a movie from a series of image files.

    This function makes it very easy to create a movie file from a
    series of image files. Several different encoding tools can be
    used, such as an HTML file, MEncoder, FFmpeg, mpeg_encode,
    ppmtompeg (Netpbm), mpeg2enc (MJPEGTools), and convert (ImageMagick),
    to combine the image files together. The encoding tool will be chosen
    automatically among these if more than one is installed on the
    machine in question (unless it is specified as a keyword argument
    to the movie function).

    Suppose we have some image files named `image_0000.eps`, `image_0001.eps`,
    `image_0002.eps`, ... Note that the zero-padding, obtained by the printf
    format `04d` in this case, ensures that the files are listed in correct
    numeric order when using a wildcard notation like `image_*.eps`.
    We want to make a movie out of these files, where each file constitutes
    a frame in the movie. This task can be accomplished by the simple call::

        movie('image_*.eps')

    The result is a movie file with a default name such as `movie.html`,
    `movie.avi`, `movie.mpeg`, or `movie.gif`, depending on the
    encoding tool chosen by the movie function. The file resides in
    the current working directory. The movie function checks a list of
    encoders and chooses the first it finds installed on the computer.

    Note: We strongly recommend to always clean up previously generated
    files for the frames in movies::

        for f in glob.glob('image_*.eps'):
            os.remove(f)

    Otherwise, there is a danger of mixing old and new files in the movie!

    One can easily specify the name of the movie file and explicitly
    specify the encoder. For example, an animated GIF movie can be
    created by::

        movie('image_*.eps', encoder='convert',
              output_file='../wave2D.gif')

    The encoder here is the convert program from the ImageMagick suite
    of image manipulation tools. The resulting movie file will be
    named 'wave2D.gif' and placed in the parent directory.

    Another convenient encoder is simply to make an HTML file that can
    play a series of image files::

        movie('image_*.png', encoder='html',
              output_file='../wave2D.html')

    Just load the output file into a web browser and play the movie.

    If you want to create an MPEG movie by using the MEncoder
    tool, you can do this with the following command::

        movie('image_*.eps', encoder='mencoder',
              output_file='/home/johannr/wave2D.mpeg',
              vcodec='mpeg2video', vbitrate=2400, qscale=4, fps=10)

    Here, we have also specified the video codec to be mpeg2video, the video
    bitrate to be 2400 kbps, the quantization scale (quality) to be 4, and
    the number of frames per second to be 10.

    Demo programs showing various use of the movie function can be
    found as the files movie_demo*.py files in the examples directory
    of the scitools source code tree.

    Suitable movie players are vlc for MPEG and AVI formats, and
    animate for animated GIF files.

    Below follows a more detailed description of the various arguments that
    are available in this function.

    Required arguments:

    input_files: Specifies the image files which will be used to make the
    movie. The argument must be given either as a string,
    e.g., 'image_*.png' or a list/tuple of strings, e.g.,
    glob.glob('image_*.png').

    Notes:

        * When using the FFmpeg or the Mpeg2enc tools, the image
          files should be given (if possible) as a string on the
          format '{1}%{2}d{3}', where the name components are as
          follows:

          - {1} filename prefix (e.g. image_)
          - {2} counting placeholder (like in C, printf, e.g. 04)
          - {3} file extension (e.g. .png or .jpg)

          Example of a correct description of the input files
          is image_%04d.png. If the input files are not given on
          the correct format, there will automatically be made
          copies of these files which will then be renamed to the
          required filename format.

        * MEncoder, FFmpeg, and Mpeg2enc supports only .jpg and
          .png image files. So, if the input files are on another
          format, there will automatically be made copies which
          in turn will be converted to the correct format.

    Optional arguments:

    output_file: Sets the name of the output movie. If not set, a default
    name like movie.avi, movie.mpeg, or movie.gif (depending
    on the output format) will be given.

    Note: When using the convert tool to combine the images,
    the extension of the file name is used to determine the
    file format of the final movie. For example, if a name like
    movie.gif is given, the resulting movie will become an
    animated gif file. Other supported movie formats are MPEG
    (.mpg, .mpeg, or .m2v) and MNG (Multiple-image Network
    Graphics).

    overwrite_output: If True, the file given in the output_file
    argument above will be overwritten without warning (if it already
    exists). The default is True.

    encoder: Sets the encoder tool to be used. Currently the following
    encoders are supported: 'html', 'mencoder', 'ffmpeg',
    'mpeg_encode', 'ppmtompeg' (from the Netpbm package),
    'mpeg2enc' (from the MJPEGTools package), and 'convert'
    (from the ImageMagick package).
    Note: 'ppmtompeg' and 'mpeg_encode' is the same tool.

    vbitrate: Sets the bit rate of the movie. The default is 800 kbps
    when using the FFmpeg and MEncoder encoders. For
    mpeg_encode, ppmtompeg, and mpeg2enc, this option is by
    default not specified. This option has no effect on the
    convert tool from ImageMagick.

    vbuffer: Sets the video buffer size. The default is to use the
    current encoding tools default video buffer size. In some
    cases it might be necessary to push this up to 500K or
    more.

    fps: Sets the number of frames per second for the final movie.
    The default is 25 fps.

    Notes:

     * The 'mpeg_encode', 'ppmtompeg', and 'mpeg2enc' tools only
       supports the following frame rates: 23.976, 24, 25,
       29.97, 30, 50, 59.94, and 60 fps.

     * Not all video codecs have support for arbitrary frame
       rates (e.g., 'mpeg1video' and 'mpeg2video').

    vcodec: Sets the video codec to be used. Some of the possible codecs
    are:

      - 'mjpeg'      - Motion JPEG
      - 'ljpeg'      - Lossless JPEG
      - 'h263'       - H.263
      - 'h263p'      - H.263+
      - 'mpeg4'      - MPEG-4 (DivX 4/5)
      - 'msmpeg4'    - DivX 3
      - 'msmpeg4v2'  - DivX 3
      - 'msmpeg4v2'  - MS MPEG4v2
      - 'wmv1'       - Windows Media Video, version 1 (AKA WMV7)
      - 'wmv2'       - Windows Media Video, version 2 (AKA WMV8)
      - 'rv10'       - an old RealVideo codec
      - 'mpeg1video' - MPEG-1 video
      - 'mpeg2video' - MPEG-2 video
      - 'huffyuv'    - HuffYUV
      - 'ffvhuff'    - nonstandard 20% smaller HuffYUV using YV12
      - 'asv1'       - ASUS Video v1
      - 'asv2'       - ASUS Video v2
      - 'ffv1'       - FFmpeg's lossless video codec

    The default codec is 'mpeg4' for mencoder/ffmpeg and
    'mpeg1video' for mpeg2enc.

    Notes:

      * Run 'ffmpeg -formats' for a longer list of available
        codecs.

      * The mencoder tool can also use the 'xvid' codec.

      * Only 'mpeg1video' and 'mpeg2video' are available when
        using the mpeg2enc tool.

      * This option has no effect when using mpeg_encode,
        ppmtompeg, or convert as the encoding tool.

    qscale: The quantization scale value (qscale) give a trade-off
    between quality and compression. A lower value means better
    quality but larger files. Larger values gives better
    compression, but worse quality. The qscale value must be an
    integer between 1 and 31. The default is to not set the
    qscale option.

    Note: When using mpeg_encode or ppmtompeg it is possible
    to have different qscale values for I, P, and B frames
    (see the iqscale, pqscale, and bqscale options below).

    qmin: Sets the minimum quantization scale value. Must be given as
    an integer between 1 and 31. The default is 2.

    qmax: Sets the maximum quantization scale value. Must be given as
    an integer between 1 and 31. The default is 31.

    iqscale: Sets the quantization scale value (see qscale) for I
    frames. This option only affects mpeg_encode and ppmtompeg.
    The default is to use the same value as in qscale. If
    qscale is not given, then 8 is the default value for
    iqscale.

    pqscale: Same as iqscale, but for P frames. If qscale is not given,
    then 10 is the default value for pqscale.

    bqscale: Same as iqscale, but for B frames. If qscale is not given,
    then 25 is the default value for bqscale.

    pattern: Sets the pattern (sequence) of I, P, and B frames. The
    default pattern is 'I' which gives good quality (but
    worse compression). Another standard sequence is
    'IBBPBBPBBPBBPBB'. This option has only an effect when
    using mpeg_encode or ppmtompeg as the encoding tool.

    size: Sets the size of the final movie. The size must be given
    as a tuple/list (e.g. (640,480)) or as a string. The
    format of the string must be 'wxh' (e.g. '320x240'), but
    the following abbreviations are also recognized:

      - 'sqcif' - 128x96
      - 'qcif'  - 176x144
      - 'cif'   - 352x288
      - '4cif'  - 704x576

    The default is to use the same size as the input images.

    aspect: Sets the aspect ratio of the movie (e.g. 4/3 or 16/9).

    Notes:

      * 'mpeg_encode' and 'ppmtompeg' only support the following
        aspect ratios: 1.0, 0.6735, 0.7031, 0.7615,0.8055,
        0.8437, 0.8935, 0.9157, 0.9815, 1.0255, 1.0695, 1.0950,
        1.1575, and 1.2015.

      * 'mpeg2enc' only supports the following aspect ratios: 1.0,
        1.33, 1.77, and 2.21.

    preferred_package: Sets whether to prefer the Netpbm package or the
    ImageMagick package if both of them are installed. Must be
    given as a string, i.e, either 'imagemagick' (default) or
    'netpbm'.

    Notes:

      * If only one of the packages is installed, then that
        package will be used.

      * If none of the packages are installed, then some
        operations might stop in lack of needed programs.

    gop_size: Sets the number of frames in a group of pictures (GOP).
    The default is to use the chosen encoding tools default
    value.

    quiet: If True, then the operations will run quietly and only
    complain on errors. The default is False.

    cleanup: If True (default), all temporary files that are created
    during the execution of the movie command will be deleted.

    force_conversion: Forces conversion of images. This is a hack that can
    be used if the encoding tool has problems reading the input
    image files. If this is set to True, the images will be
    converted even if they are in a format recognized by the
    encoding tool. The default is False.

    Known issues:

      * JPEG images created by the Vtk backend does not seem to work with
        the MEncoder and FFmpeg tools. This can be fixed by setting the
        force_conversion argument to True. This will force conversion of the
        JPEG files to PNG files which in turn should successfully create the
        movie.

      * Aspect ratio in mpeg_encode does not seem to work.
    """
    me = MovieEncoder(input_files, **kwargs)
    me.encode()


if __name__ == '__main__':
    pass



"""Test functions for grid command."""

from easyviztest import *


class test_grid_basic(EasyvizTestCase):
    def check_grid_on(self):
        x,y,v,w = self.get_line_data()
        plot(x,y)
        grid('on')
        title("plot(..);grid('on')")
        n()
        
    def check_grid_on_kwarg(self):
        x,y,v,w = self.get_line_data()
        plot(x,y,grid='on')
        title("plot(..,grid='on')")
        n()
        
    def check_grid_off(self):
        x,y,v,w = self.get_line_data()
        plot(x,y)
        grid('on')
        grid('off')
        title("plot(..);grid('on');grid('off')")
        n()
        
    def check_grid_toggle_on(self):
        x,y,v,w = self.get_line_data()
        plot(x,y)
        grid()
        title("plot(..);grid() (toggle on)")
        n()

    def check_grid_toggle_off(self):
        x,y,v,w = self.get_line_data()
        plot(x,y)
        grid()
        grid()
        title("plot(..);grid();grid() (toggle off)")
        n()

    def check_grid_on_specify_axis(self):
        x,y,v,w = self.get_line_data()
        plot(x,y)
        grid(gca(),'on')
        title("plot(..);grid(gca(),'on')")
        n()

    def check_grid_toggle_on_specify_axis(self):
        x,y,v,w = self.get_line_data()
        plot(x,y)
        grid(gca())
        title("plot(..);grid(gca()) (toggle on)")
        n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_grid_basic,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")



"""Test functions for caxis command."""

from easyviztest import *


class test_caxis_basic(EasyvizTestCase):
    def check_caxis(self):
        x,y,xv,yv,values = self.get_2D_data()
        pcolor(xv,yv,values,colorbar='on')
        caxis(-2,2)
        title("pcolor(..);caxis(-2,2)")
        n()

    def check_caxis_list(self):
        x,y,xv,yv,values = self.get_2D_data()
        pcolor(xv,yv,values,colorbar='on')
        caxis([-5,5])
        title("pcolor(..);caxis([-5,5])")
        n()

    def check_caxis_kwarg(self):
        x,y,xv,yv,values = self.get_2D_data()
        pcolor(xv,yv,values,colorbar='on',caxis=[0,8])
        title("pcolor(..,caxis=[0,8])")
        n()

    def check_caxis_auto(self):
        x,y,xv,yv,values = self.get_2D_data()
        setp(show=False)
        pcolor(xv,yv,values,colorbar='on')
        caxis(-2,2)
        caxis('auto')
        title("pcolor(..);caxis(-2,2);caxis('auto')")
        setp(show=True)
        show()
        n()

    def check_caxis_manual(self):
        x,y,xv,yv,values = self.get_2D_data()
        setp(show=False)
        surf(xv,yv,xv**2+yv**2,colorbar='on')
        caxis('manual')
        hold('on')
        surf(xv,yv,xv+yv)
        title("surf(..);caxis('manual');hold('on');surf(..)")
        hold('off')
        setp(show=True)
        show()
        n()

    def check_caxis_list(self):
        x,y,xv,yv,values = self.get_2D_data()
        pcolor(xv,yv,values,colorbar='on')
        caxis(gca(), [-8,1])
        title("pcolor(..);caxis(gca(), [-8,1])")
        n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_caxis_basic,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")



"""Test functions for view command."""

from easyviztest import *


class test_view(EasyvizTestCase):
    def check_view_default_2D_view(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        view(2)
        title('surf(..);view(2)')
        n()

    def check_view_default_2D_view_kwarg(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values,view=2)
        title('surf(..,view=2)')
        n()
    
    def check_view_default_3D_view(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        view(3)
        title('surf(..);view(3)')
        n()
    
    def check_view_default_3D_view_kwarg(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values,view=3)
        title('surf(..,view=3)')
        n()

    def check_view_azimuth_and_elevation(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        view(40,70)
        title('surf(..);view(40,70)')
        n()

    def check_view_azimuth_and_elevation_list(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        view([25,60])
        title('surf(..);view([25,60])')
        n()

    def check_view_azimuth_and_elevation_kwarg(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values,view=(30,30))
        title('surf(..,view=(30,30))')
        n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_view,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")
